﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278._005010X215;

namespace X12Engine.BLL.X12._278._005010X215
{
    public static class MessageManager
    {
        public static Message Parse(string message)
        {
            Message objReturn = null;

            string[] loops = Utils.Split(message, "HL" + Settings.Data_Element_Separator);
            if (loops != null)
            {
                objReturn = new Message();
                string[] hierarchicalIdList = null;

                foreach (string loop in loops)
                {
                    string[] segments = loop.Split(Settings.Segment_Terminator);
                    if (segments != null)
                    {
                        foreach (string segment in segments)
                        {
                            if (string.IsNullOrEmpty(segment)) continue;

                            string[] elements = segment.Split(Settings.Data_Element_Separator);
                            if (elements != null)
                            {
                                string segmentId = elements[0].ToUpper();
                                switch (segmentId)
                                {
                                    case "ST":
                                        objReturn.TransactionSetHeader = STManager.Parse(segment);
                                        break;

                                    case "BHT":
                                        objReturn.BeginningOfHierarchicalTransaction = BHTManager.Parse(segment);
                                        break;

                                    case "SE":
                                        objReturn.TransactionSetTrailer = SEManager.Parse(segment);
                                        break;

                                    case "HL":

                                        string hierarchicalIdNumber = string.Empty;
                                        string hierarchicalLevelCode = string.Empty;

                                        if (elements.Length > 1) hierarchicalIdNumber = elements[1];
                                        if (elements.Length > 3) hierarchicalLevelCode = elements[3].ToUpper();

                                        //FS-278-3.0-051 //FS-278-3.0-068 //FS-278-3.0-087 //FS-278-3.0-111 //FS-278-3.0-139
                                        if (Utils.Contains(hierarchicalIdList, hierarchicalIdNumber))
                                        {
                                            //TODO: Return AAA and/or Log Error???
                                        }
                                        else
                                        {
                                            Utils.AddToArray(ref hierarchicalIdList, hierarchicalIdNumber);
                                        }

                                        if (hierarchicalLevelCode == "20") objReturn.UmoDetail = UmoDetailManager.Parse(loop);
                                        else if (hierarchicalLevelCode == "21") objReturn.RequesterDetail = RequesterDetailManager.Parse(loop);
                                        else if (hierarchicalLevelCode == "22")
                                        {
                                            if (objReturn.SubscriberDetail == null) objReturn.SubscriberDetail = new List<SubscriberDetail>();
                                            objReturn.SubscriberDetail.Add(SubscriberDetailManager.Parse(loop));
                                        }
                                        else if (hierarchicalLevelCode == "23")
                                        {
                                            if (objReturn.DependentDetail == null) objReturn.DependentDetail = new List<DependentDetail>();
                                            objReturn.DependentDetail.Add(DependentDetailManager.Parse(loop));
                                        }
                                        else if (hierarchicalLevelCode == "EV")
                                        {
                                            if (objReturn.PatientEventDetail == null) objReturn.PatientEventDetail = new List<PatientEventDetail>();
                                            objReturn.PatientEventDetail.Add(PatientEventDetailManager.Parse(loop));
                                        }
                                        else if (hierarchicalLevelCode == "SS")
                                        {
                                            if (objReturn.ServiceDetail == null) objReturn.ServiceDetail = new List<ServiceDetail>();
                                            objReturn.ServiceDetail.Add(ServiceDetailManager.Parse(loop));
                                        }

                                        break;

                                    default:
                                        break;
                                }
                            }
                        }
                    }
                }
            }

            return objReturn;
        }
    }
}
