﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278._005010X217;

namespace X12Engine.BLL.X12._278._005010X217
{
   public static class PatientEventDetailManager
   {
      public static PatientEventDetail Parse(string loop)
      {
         PatientEventDetail objReturn = new PatientEventDetail();

         string[] segments = loop.Split(Settings.Segment_Terminator);
         if (segments != null)
         {
            bool contactEntityFound = false;
            bool transportEntityFound = false;

            EntityBase patientEventProviderName = null;
            EntityBase patientEventTransportInformation = null;

            foreach (string segment in segments)
            {
               if (string.IsNullOrEmpty(segment)) continue;

               string[] elements = segment.Split(Settings.Data_Element_Separator);
               if (elements != null && !string.IsNullOrEmpty(elements[0]))
               {
                  string segmentId = elements[0].ToUpper();
                  string qualifier = elements[1].ToUpper();

                  if (segmentId == "HL")
                     objReturn.PatientEventLevel = HLManager.Parse(segment);
                  else if (segmentId == "TRN")
                  {
                     if (objReturn.PatientEventTraceNumber == null) objReturn.PatientEventTraceNumber = new List<TRN>();
                     objReturn.PatientEventTraceNumber.Add(TRNManager.Parse(segment));
                  }
                  //else if (segmentId == "AAA" && !entitySegmentFound)
                  //{
                  //      if (objReturn.PatientEventRequestValidation == null) objReturn.PatientEventRequestValidation = new List<AAA>();
                  //      objReturn.PatientEventRequestValidation.Add(AAAManager.Parse(segment));
                  //}
                  else if (segmentId == "UM")
                        objReturn.HealthCareServicesReviewInformation = UMManager.Parse(segment);
                  else if (segmentId == "HCR")
                        objReturn.HealthCareServicesReview = HCRManager.Parse(segment);
                  else if (segmentId == "REF" && qualifier == "BB")
                        objReturn.PreviousReviewAuthorizationNumber = REFManager.Parse(segment);
                  else if (segmentId == "REF" && qualifier == "NT")
                        objReturn.PreviousReviewAdministrativeReferenceNumber = REFManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "439")
                        objReturn.AccidentDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "484")
                        objReturn.LastMenstrualPeriodDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "ABC")
                        objReturn.EstimatedDateOfBirth = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "431")
                        objReturn.OnsetOfCurrentSymptomsOrIllnessDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "AAH")        //----------------------------------- Event Date  xxx
                     objReturn.EventDate = DTPManager.Parse(segment);

                  else if (segmentId == "DTP" && qualifier == "435")
                        objReturn.AdmissionDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "096")
                        objReturn.DischargeDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "102")
                        objReturn.CertificationIssueDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "036")
                        objReturn.CertificationExpirationDate = DTPManager.Parse(segment);
                  else if (segmentId == "DTP" && qualifier == "007")
                        objReturn.CertificationEffectiveDate = DTPManager.Parse(segment);
                  else if (segmentId == "HI")
                        objReturn.PatientDiagnosis = HIManager.Parse(segment);
                  else if (segmentId == "HSD")
                        objReturn.HealthCareServicesDelivery = HSDManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "07")
                        objReturn.AmbulanceCertificationInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "08")
                        objReturn.ChiropracticCertificationInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "09")
                        objReturn.DurableMedicalEquipmentInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "11")
                        objReturn.OxygenTherapyCertificationInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "75")
                        objReturn.FunctionalLimitationsInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "76")
                        objReturn.ActivitiesPermittedInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CRC" && qualifier == "77")
                        objReturn.MentalStatusInformation = CRCManager.Parse(segment);
                  else if (segmentId == "CL1")
                        objReturn.InstitutionalClaimCode = CL1Manager.Parse(segment);
                  else if (segmentId == "CR1")
                        objReturn.AmbulanceTransportInformation = CR1Manager.Parse(segment);
                  else if (segmentId == "CR2")
                        objReturn.SpinalManipulationServiceInformation = CR2Manager.Parse(segment);
                  else if (segmentId == "CR5")
                        objReturn.HomeOxygenTherapyInformation = CR5Manager.Parse(segment);
                  else if (segmentId == "CR6")
                        objReturn.HomeHealthCareInformation = CR6Manager.Parse(segment);
                  else if (segmentId == "PWK")
                  {
                        if (objReturn.AdditionalPatientInformation == null) objReturn.AdditionalPatientInformation = new List<PWK>();
                        objReturn.AdditionalPatientInformation.Add(PWKManager.Parse(segment));
                  }
                  else if (segmentId == "MSG")
                        objReturn.MessageText = MSGManager.Parse(segment);
                  else if (Utils.IsEntitySegment(segmentId))
                  {
                     if (Utils.IsTransportEntity(qualifier) || transportEntityFound)
                     {
                        transportEntityFound = true;
                        contactEntityFound = false;

                        if (objReturn.PatientEventTransportInformation == null) objReturn.PatientEventTransportInformation = new List<EntityBase>();

                        if (segmentId == "NM1")
                        {
                           if (patientEventTransportInformation != null) objReturn.PatientEventTransportInformation.Add(patientEventTransportInformation);
                           patientEventTransportInformation = new EntityBase();
                        }

                        patientEventTransportInformation = EntityBaseManager.Parse(segment, patientEventTransportInformation);
                     }
                     else if (Utils.IsContactEntity(qualifier) || contactEntityFound)
                     {
                        contactEntityFound = true;
                        transportEntityFound = false;

                        objReturn.AdditionalPatientInformationContactName = EntityBaseManager.Parse(segment, objReturn.AdditionalPatientInformationContactName);
                     }
                     else
                     {
                        contactEntityFound = false;
                        transportEntityFound = false;

                        if (objReturn.PatientEventProviderName == null) objReturn.PatientEventProviderName = new List<EntityBase>();

                        if (segmentId == "NM1")
                        {
                           if (patientEventProviderName != null) objReturn.PatientEventProviderName.Add(patientEventProviderName);
                           patientEventProviderName = new EntityBase();
                        }

                        patientEventProviderName = EntityBaseManager.Parse(segment, patientEventProviderName);
                     }                            
                  }
                  else
                     objReturn.Other = DetailBaseManager.Parse(segment, objReturn.Other);
               }
            }

            if (patientEventProviderName != null) objReturn.PatientEventProviderName.Add(patientEventProviderName);
            if (patientEventTransportInformation != null) objReturn.PatientEventTransportInformation.Add(patientEventTransportInformation);
         }

         return objReturn;
      }
   }
}
