﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X12Engine.BO.X12._278.IdentifierCodes;

namespace X12Engine.BLL.X12._278
{


	public static class AAAManager
	{
		public static List<AAA> UmoDetailAAA = new List<AAA>();
		public static List<AAA> RequesterDetailAAA = new List<AAA>();
		public static List<AAA> SubscriberDetailAAA = new List<AAA>();
		public static List<AAA> DependentDetailAAA = new List<AAA>();
		public static List<AAA> PatientEventDetailAAA = new List<AAA>();
		public static List<AAA> ServiceDetailAAA = new List<AAA>();

	
		// HL1    
		public static void AddUmoDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.UmoDetailAAA.Add(aaa);
			Utils.LogEvent("AddUmoDetailError");
		}

		// HL2
		public static void AddRequesterDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.RequesterDetailAAA.Add(aaa);
			Utils.LogEvent("AddRequesterDetailError");
		}

		// HL3
		public static void AddSubscriberDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.SubscriberDetailAAA.Add(aaa);
			Utils.LogEvent("AddSubscriberDetailError");
		}


		// NA
		public static void AddDependentDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.DependentDetailAAA.Add(aaa);
			Utils.LogEvent("AddDependentDetailError");
		}



		// HL4
		public static void AddPatientEventDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.PatientEventDetailAAA.Add(aaa);
			Utils.LogEvent("AddPatientEventDetailError");
		}

		// NA
		public static void AddServiceDetailError(string YesNoConditionOrResponseCode, string RejectReasonCode, string FollowUpActionCode, string ErrorDescription = "")
		{
			AAA aaa = new AAA();
			aaa.YesNoConditionOrResponseCode = new IDENTIFIER(YesNoConditionOrResponseCode);
			aaa.RejectReasonCode = new IDENTIFIER(RejectReasonCode);
			aaa.FollowUpActionCode = new IDENTIFIER(FollowUpActionCode);
			AAAManager.ServiceDetailAAA.Add(aaa);
			Utils.LogEvent("AddServiceDetailError");
		}

	}




	public static class HLManager
	{
		public static HL Parse(string segment)
		{
			HL objReturn = new HL();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.HierarchicalIdNumber = elements[1];
				if (elements.Length > 2) objReturn.HierarchicalParentIdNumber = elements[2];
				if (elements.Length > 3) objReturn.HierarchicalLevelCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("HierarchicalLevelCode", objReturn.HierarchicalLevelCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.HierarchicalChildCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("HierarchicalChildCode", objReturn.HierarchicalChildCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class TRNManager
	{
		public static TRN Parse(string segment)
		{
			TRN objReturn = new TRN();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.TraceTypeCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("TraceTypeCode", objReturn.TraceTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.ReferenceIdentification1 = elements[2];
				if (elements.Length > 3) objReturn.OriginatingCompanyIdentifier = elements[3];
				if (elements.Length > 4) objReturn.ReferenceIdentification2 = elements[4];
			}

			return objReturn;
		}
	}


	public static class UMManager
	{
		public static UM Parse(string segment)
		{
			UM objReturn = new UM();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.RequestCategoryCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("RequestCategoryCode", objReturn.RequestCategoryCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.CertificationTypeCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("CertificationTypeCode", objReturn.CertificationTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.ServiceTypeCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("ServiceTypeCode", objReturn.ServiceTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.HealthCareServiceLocationInformation = new HEALTH_CARE_SERVICE_LOCATION_INFORMATION(elements[4]);
				if (elements.Length > 5) objReturn.RelatedCausesInformation = new RELATED_CAUSES_INFORMATION(elements[5]);
				if (elements.Length > 6) objReturn.LevelOfServiceCode = new IDENTIFIER(elements[6]);
				if (!Code.Validate("LevelOfServiceCode", objReturn.LevelOfServiceCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 7) objReturn.CurrentHealthConditionCode = new IDENTIFIER(elements[7]);
				if (!Code.Validate("CurrentHealthConditionCode", objReturn.CurrentHealthConditionCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.PrognosisCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("PrognosisCode", objReturn.PrognosisCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.ReleaseOfInformationCode = new IDENTIFIER(elements[9]);
				if (!Code.Validate("ReleaseOfInformationCode", objReturn.ReleaseOfInformationCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.DelayReasonCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("DelayReasonCode", objReturn.DelayReasonCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class HCRManager
	{
		public static HCR Parse(string segment)
		{
			HCR objReturn = new HCR();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ActionCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("ActionCode", objReturn.ActionCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.ReferenceIdentification = elements[2];
				if (elements.Length > 3) objReturn.IndustryCode = elements[3];
				if (elements.Length > 4) objReturn.YesNoConditionOrResponseCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class REFManager
	{
		public static REF Parse(string segment)
		{
			REF objReturn = new REF();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ReferenceIdentificationQualifier = new IDENTIFIER(elements[1]);
				if (!Code.Validate("ReferenceIdentificationQualifier", objReturn.ReferenceIdentificationQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.ReferenceIdentification = elements[2];
				if (elements.Length > 3) objReturn.Description = elements[3];
				if (elements.Length > 4) objReturn.ReferenceIdentifier = new REFERENCE_IDENTIFIER(elements[4]);
			}

			return objReturn;
		}
	}

	public static class DTPManager
	{
		public static DTP Parse(string segment)
		{
			DTP objReturn = new DTP();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.DateTimeQualifier = new IDENTIFIER(elements[1]);
				//if (!Code.Validate("DateTimeQualifier", objReturn.DateTimeQualifier))
				//{                
				//}

				if (elements.Length > 2) objReturn.DateTimePeriodFormatQualifier = new IDENTIFIER(elements[2]);

				//if (!Code.Validate("DateTimePeriodFormatQualifier", objReturn.DateTimePeriodFormatQualifier))
				//{              
				//}

				if (elements.Length > 3) objReturn.DateTimePeriod = elements[3];
			}

			return objReturn;
		}
	}

	public static class HIManager
	{
		public static HI Parse(string segment)
		{
			HI objReturn = new HI();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.HealthCareCodeInformation1 = new HEALTH_CARE_CODE_INFORMATION(elements[1]);
				if (elements.Length > 2) objReturn.HealthCareCodeInformation2 = new HEALTH_CARE_CODE_INFORMATION(elements[2]);
				if (elements.Length > 3) objReturn.HealthCareCodeInformation3 = new HEALTH_CARE_CODE_INFORMATION(elements[3]);
				if (elements.Length > 4) objReturn.HealthCareCodeInformation4 = new HEALTH_CARE_CODE_INFORMATION(elements[4]);
				if (elements.Length > 5) objReturn.HealthCareCodeInformation5 = new HEALTH_CARE_CODE_INFORMATION(elements[5]);
				if (elements.Length > 6) objReturn.HealthCareCodeInformation6 = new HEALTH_CARE_CODE_INFORMATION(elements[6]);
				if (elements.Length > 7) objReturn.HealthCareCodeInformation7 = new HEALTH_CARE_CODE_INFORMATION(elements[7]);
				if (elements.Length > 8) objReturn.HealthCareCodeInformation8 = new HEALTH_CARE_CODE_INFORMATION(elements[8]);
				if (elements.Length > 9) objReturn.HealthCareCodeInformation9 = new HEALTH_CARE_CODE_INFORMATION(elements[9]);
				if (elements.Length > 10) objReturn.HealthCareCodeInformation10 = new HEALTH_CARE_CODE_INFORMATION(elements[10]);
				if (elements.Length > 11) objReturn.HealthCareCodeInformation11 = new HEALTH_CARE_CODE_INFORMATION(elements[11]);
				if (elements.Length > 12) objReturn.HealthCareCodeInformation12 = new HEALTH_CARE_CODE_INFORMATION(elements[12]);
			}

			return objReturn;
		}
	}

	public static class SV1Manager
	{
		public static SV1 Parse(string segment)
		{
			SV1 objReturn = new SV1();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(elements[1]);
				if (elements.Length > 2) objReturn.MonetaryAmount1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.Quantity = Utils.ParseDecimal(elements[4]);
				if (elements.Length > 5) objReturn.FacilityCodeValue = elements[5];
				if (elements.Length > 6) objReturn.ServiceTypeCode = new IDENTIFIER(elements[6]);
				if (!Code.Validate("ServiceTypeCode", objReturn.ServiceTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 7) objReturn.CompositeDiagnosisCodePointer = new COMPOSITE_DIAGNOSIS_CODE_POINTER(elements[7]);
				if (elements.Length > 8) objReturn.MonetaryAmount2 = Utils.ParseDecimal(elements[8]);
				if (elements.Length > 9) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[9]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.MultipleProcedureCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("MultipleProcedureCode", objReturn.MultipleProcedureCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[11]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 12) objReturn.YesNoConditionOrResponseCode3 = new IDENTIFIER(elements[12]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode3))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 13) objReturn.ReviewCode = new IDENTIFIER(elements[13]);
				if (!Code.Validate("ReviewCode", objReturn.ReviewCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 14) objReturn.NationalOrLocalAssignedReviewValue = elements[14];
				if (elements.Length > 15) objReturn.CopayStatusCode = new IDENTIFIER(elements[15]);
				if (!Code.Validate("CopayStatusCode", objReturn.CopayStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 16) objReturn.HealthCareProfessionalShortageAreaCode = new IDENTIFIER(elements[16]);
				if (!Code.Validate("HealthCareProfessionalShortageAreaCode", objReturn.HealthCareProfessionalShortageAreaCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 17) objReturn.ReferenceIdentification = elements[17];
				if (elements.Length > 18) objReturn.PostalCode = new IDENTIFIER(elements[18]);
				if (!Code.Validate("PostalCode", objReturn.PostalCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 19) objReturn.MonetaryAmount3 = Utils.ParseDecimal(elements[19]);
				if (elements.Length > 20) objReturn.LevelOfCareCode = new IDENTIFIER(elements[20]);
				if (!Code.Validate("LevelOfCareCode", objReturn.LevelOfCareCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 21) objReturn.ProviderAgreementCode = new IDENTIFIER(elements[21]);
				if (!Code.Validate("ProviderAgreementCode", objReturn.ProviderAgreementCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class SV2Manager
	{
		public static SV2 Parse(string segment)
		{
			SV2 objReturn = new SV2();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ProductServiceId = elements[1];
				if (elements.Length > 2) objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(elements[2]);
				if (elements.Length > 3) objReturn.MonetaryAmount1 = Utils.ParseDecimal(elements[3]);
				if (elements.Length > 4) objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.Quantity = Utils.ParseDecimal(elements[5]);
				if (elements.Length > 6) objReturn.UnitRate = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.MonetaryAmount2 = Utils.ParseDecimal(elements[7]);
				if (elements.Length > 8) objReturn.YesNoConditionOrResponseCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.NursingHomeResidentialStatusCode = new IDENTIFIER(elements[9]);
				if (!Code.Validate("NursingHomeResidentialStatusCode", objReturn.NursingHomeResidentialStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.LevelOfCareCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("LevelOfCareCode", objReturn.LevelOfCareCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class SV3Manager
	{
		public static SV3 Parse(string segment)
		{
			SV3 objReturn = new SV3();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CompositeMedicalProcedureIdentifier = new COMPOSITE_MEDICAL_PROCEDURE_IDENTIFIER(elements[1]);
				if (elements.Length > 2) objReturn.MonetaryAmount = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.FacilityCodeValue = elements[3];
				if (elements.Length > 4) objReturn.OralCavityDesignation = new ORAL_CAVITY_DESIGNATION(elements[4]);
				if (elements.Length > 5) objReturn.ProsthesisCrownOrInlayCode = new IDENTIFIER(elements[5]);
				if (!Code.Validate("ProsthesisCrownOrInlayCode", objReturn.ProsthesisCrownOrInlayCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.Quantity = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Description = elements[7];
				if (elements.Length > 8) objReturn.CopayStatusCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("CopayStatusCode", objReturn.CopayStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.ProviderAgreementCode = new IDENTIFIER(elements[9]);
				if (!Code.Validate("ProviderAgreementCode", objReturn.ProviderAgreementCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.YesNoConditionOrResponseCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.CompositeDiagnosisCodePointer = new COMPOSITE_DIAGNOSIS_CODE_POINTER(elements[11]);
			}

			return objReturn;
		}
	}

	public static class TOOManager
	{
		public static TOO Parse(string segment)
		{
			TOO objReturn = new TOO();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CodeListQualifierCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("CodeListQualifierCode", objReturn.CodeListQualifierCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.IndustryCode = elements[2];
				if (elements.Length > 3) objReturn.ToothSurface = new TOOTH_SURFACE(elements[3]);
			}

			return objReturn;
		}
	}

	public static class HSDManager
	{
		public static HSD Parse(string segment)
		{
			HSD objReturn = new HSD();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.QuantityQualifier = new IDENTIFIER(elements[1]);
				if (!Code.Validate("QuantityQualifier", objReturn.QuantityQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.Quantity = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.SampleSelectionModulus = Utils.ParseDecimal(elements[4]);
				if (elements.Length > 5) objReturn.TimePeriodQualifier = new IDENTIFIER(elements[5]);
				if (!Code.Validate("TimePeriodQualifier", objReturn.TimePeriodQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.NumberOfPeriods = Utils.ParseInt(elements[6]);
				if (elements.Length > 7) objReturn.ShipDeliveryOrCalendarPatternCode = new IDENTIFIER(elements[7]);
				if (!Code.Validate("ShipDeliveryOrCalendarPatternCode", objReturn.ShipDeliveryOrCalendarPatternCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.ShipDeliveryPatternTimeCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("ShipDeliveryPatternTimeCode", objReturn.ShipDeliveryPatternTimeCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class CRCManager
	{
		public static CRC Parse(string segment)
		{
			CRC objReturn = new CRC();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CodeCategory = new IDENTIFIER(elements[1]);
				if (!Code.Validate("CodeCategory", objReturn.CodeCategory))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.YesNoConditionOrResponseCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.ConditionIndicator1 = new IDENTIFIER(elements[3]);
				if (!Code.Validate("ConditionIndicator", objReturn.ConditionIndicator1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.ConditionIndicator2 = new IDENTIFIER(elements[4]);
				if (!Code.Validate("ConditionIndicator", objReturn.ConditionIndicator2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.ConditionIndicator3 = new IDENTIFIER(elements[5]);
				if (!Code.Validate("ConditionIndicator", objReturn.ConditionIndicator3))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.ConditionIndicator4 = new IDENTIFIER(elements[6]);
				if (!Code.Validate("ConditionIndicator", objReturn.ConditionIndicator4))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 7) objReturn.ConditionIndicator5 = new IDENTIFIER(elements[7]);
				if (!Code.Validate("ConditionIndicator", objReturn.ConditionIndicator5))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class CL1Manager
	{
		public static CL1 Parse(string segment)
		{
			CL1 objReturn = new CL1();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.AdmissionTypeCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("AdmissionTypeCode", objReturn.AdmissionTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.AdmissionSourceCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("AdmissionSourceCode", objReturn.AdmissionSourceCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.PatientStatusCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("PatientStatusCode", objReturn.PatientStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.NursingHomeResidentialStatusCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("NursingHomeResidentialStatusCode", objReturn.NursingHomeResidentialStatusCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class CR1Manager
	{
		public static CR1 Parse(string segment)
		{
			CR1 objReturn = new CR1();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.UnitOrBasisForMeasurementCode1 = new IDENTIFIER(elements[1]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.Weight = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.AmbulanceTransportCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("AmbulanceTransportCode", objReturn.AmbulanceTransportCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.AmbulanceTransportReasonCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("AmbulanceTransportReasonCode", objReturn.AmbulanceTransportReasonCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.UnitOrBasisForMeasurementCode2 = new IDENTIFIER(elements[5]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.Quantity = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.AddressInformation1 = elements[7];
				if (elements.Length > 8) objReturn.AddressInformation2 = elements[8];
				if (elements.Length > 9) objReturn.Description1 = elements[9];
				if (elements.Length > 10) objReturn.Description2 = elements[10];
			}

			return objReturn;
		}
	}

	public static class CR2Manager
	{
		public static CR2 Parse(string segment)
		{
			CR2 objReturn = new CR2();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.Count = Utils.ParseInt(elements[1]);
				if (elements.Length > 2) objReturn.Quantity1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.SubluxationLevelCode1 = new IDENTIFIER(elements[3]);
				if (!Code.Validate("SubluxationLevelCode", objReturn.SubluxationLevelCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.SubluxationLevelCode2 = new IDENTIFIER(elements[4]);
				if (!Code.Validate("SubluxationLevelCode", objReturn.SubluxationLevelCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.UnitOrBasisForMeasurementCode = new IDENTIFIER(elements[5]);
				if (!Code.Validate("UnitOrBasisForMeasurementCode", objReturn.UnitOrBasisForMeasurementCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.Quantity2 = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Quantity3 = Utils.ParseDecimal(elements[7]);
				if (elements.Length > 8) objReturn.NatureOfConditionCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("NatureOfConditionCode", objReturn.NatureOfConditionCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[9]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.Description1 = elements[10];
				if (elements.Length > 11) objReturn.Description2 = elements[11];
				if (elements.Length > 12) objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[12]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode2))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class CR4Manager
	{
		public static CR4 Parse(string segment)
		{
			CR4 objReturn = new CR4();

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class CR5Manager
	{
		public static CR5 Parse(string segment)
		{
			CR5 objReturn = new CR5();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CertificationTypeCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("CertificationTypeCode", objReturn.CertificationTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.Quantity1 = Utils.ParseDecimal(elements[2]);
				if (elements.Length > 3) objReturn.OxygenEquipmentTypeCode1 = new IDENTIFIER(elements[3]);
				if (!Code.Validate("OxygenEquipmentTypeCode", objReturn.OxygenEquipmentTypeCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.OxygenEquipmentTypeCode2 = new IDENTIFIER(elements[4]);
				if (!Code.Validate("OxygenEquipmentTypeCode", objReturn.OxygenEquipmentTypeCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.Description1 = elements[5];
				if (elements.Length > 6) objReturn.Quantity2 = Utils.ParseDecimal(elements[6]);
				if (elements.Length > 7) objReturn.Quantity3 = Utils.ParseDecimal(elements[7]);
				if (elements.Length > 8) objReturn.Quantity4 = Utils.ParseDecimal(elements[8]);
				if (elements.Length > 9) objReturn.Description2 = elements[9];
				if (elements.Length > 10) objReturn.Quantity5 = Utils.ParseDecimal(elements[10]);
				if (elements.Length > 11) objReturn.Quantity6 = Utils.ParseDecimal(elements[11]);
				if (elements.Length > 12) objReturn.OxygenTestConditionCode1 = new IDENTIFIER(elements[12]);
				if (!Code.Validate("OxygenTestConditionCode", objReturn.OxygenTestConditionCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 13) objReturn.OxygenTestConditionCode2 = new IDENTIFIER(elements[13]);
				if (!Code.Validate("OxygenTestConditionCode", objReturn.OxygenTestConditionCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 14) objReturn.OxygenTestConditionCode3 = new IDENTIFIER(elements[14]);
				if (!Code.Validate("OxygenTestConditionCode", objReturn.OxygenTestConditionCode3))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 15) objReturn.OxygenTestConditionCode4 = new IDENTIFIER(elements[15]);
				if (!Code.Validate("OxygenTestConditionCode", objReturn.OxygenTestConditionCode4))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 16) objReturn.Quantity7 = Utils.ParseDecimal(elements[16]);
				if (elements.Length > 17) objReturn.OxygenDeliverySystemCode = new IDENTIFIER(elements[17]);
				if (!Code.Validate("OxygenDeliverySystemCode", objReturn.OxygenDeliverySystemCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 18) objReturn.OxygenEquipmentTypeCode3 = new IDENTIFIER(elements[18]);
				if (!Code.Validate("OxygenEquipmentTypeCode", objReturn.OxygenEquipmentTypeCode3))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class CR6Manager
	{
		public static CR6 Parse(string segment)
		{
			CR6 objReturn = new CR6();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.PrognosisCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("PrognosisCode", objReturn.PrognosisCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2)
				{
					objReturn.Date1 = Utils.ParseDate(elements[2], "yyyyMMdd");
					objReturn.Date1String = elements[2];
				}
				if (elements.Length > 3) objReturn.DateTimePeriodFormatQualifier1 = new IDENTIFIER(elements[3]);
				if (!Code.Validate("DateTimePeriodFormatQualifier", objReturn.DateTimePeriodFormatQualifier1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.DateTimePeriod1 = elements[4];
				if (elements.Length > 5)
				{
					objReturn.Date2 = Utils.ParseDate(elements[5], "yyyyMMdd");
					objReturn.Date2String = elements[5];
				}
				if (elements.Length > 6) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[6]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 7) objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[7]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.CertificationTypeCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("CertificationTypeCode", objReturn.CertificationTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9)
				{
					objReturn.Date3 = Utils.ParseDate(elements[9], "yyyyMMdd");
					objReturn.Date3String = elements[9];
				}
				if (elements.Length > 10) objReturn.ProductServiceIdQualifier = new IDENTIFIER(elements[10]);
				if (!Code.Validate("ProductServiceIdQualifier", objReturn.ProductServiceIdQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.MedicalCodeValue = elements[11];
				if (elements.Length > 12)
				{
					objReturn.Date4 = Utils.ParseDate(elements[12], "yyyyMMdd");
					objReturn.Date4String = elements[12];
				}
				if (elements.Length > 13)
				{
					objReturn.Date5 = Utils.ParseDate(elements[13], "yyyyMMdd");
					objReturn.Date5String = elements[13];
				}
				if (elements.Length > 14)
				{
					objReturn.Date6 = Utils.ParseDate(elements[14], "yyyyMMdd");
					objReturn.Date6String = elements[14];
				}
				if (elements.Length > 15) objReturn.DateTimePeriodFormatQualifier2 = new IDENTIFIER(elements[15]);
				if (!Code.Validate("DateTimePeriodFormatQualifier", objReturn.DateTimePeriodFormatQualifier2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 16) objReturn.DateTimePeriod2 = elements[16];
				if (elements.Length > 17) objReturn.PatientLocationCode = new IDENTIFIER(elements[17]);
				if (!Code.Validate("PatientLocationCode", objReturn.PatientLocationCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 18)
				{
					objReturn.Date7 = Utils.ParseDate(elements[18], "yyyyMMdd");
					objReturn.Date7String = elements[18];
				}
				if (elements.Length > 19)
				{
					objReturn.Date8 = Utils.ParseDate(elements[19], "yyyyMMdd");
					objReturn.Date8String = elements[19];
				}
				if (elements.Length > 20)
				{
					objReturn.Date9 = Utils.ParseDate(elements[20], "yyyyMMdd");
					objReturn.Date9String = elements[20];
				}
				if (elements.Length > 21)
				{
					objReturn.Date10 = Utils.ParseDate(elements[21], "yyyyMMdd");
					objReturn.Date10String = elements[21];
				}
			}

			return objReturn;
		}
	}

	public static class CR7Manager
	{
		public static CR7 Parse(string segment)
		{
			CR7 objReturn = new CR7();

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class CR8Manager
	{
		public static CR8 Parse(string segment)
		{
			CR8 objReturn = new CR8();

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class PWKManager
	{
		public static PWK Parse(string segment)
		{
			PWK objReturn = new PWK();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ReportTypeCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("ReportTypeCode", objReturn.ReportTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.ReportTransmissionCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("ReportTransmissionCode", objReturn.ReportTransmissionCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.ReportCopiesNeeded = Utils.ParseInt(elements[3]);
				if (elements.Length > 4) objReturn.EntityIdentifierCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("EntityIdentifierCode", objReturn.EntityIdentifierCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.IdentificationCodeQualifier = new IDENTIFIER(elements[5]);
				if (!Code.Validate("IdentificationCodeQualifier", objReturn.IdentificationCodeQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.IdentificationCode = elements[6];
				if (elements.Length > 7) objReturn.Description = elements[7];
				if (elements.Length > 8) objReturn.ActionsIndicated = new ACTIONS_INDICATED(elements[8]);
				if (elements.Length > 9) objReturn.RequestCategoryCode = new IDENTIFIER(elements[9]);
				if (!Code.Validate("RequestCategoryCode", objReturn.RequestCategoryCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class MSGManager
	{
		public static MSG Parse(string segment)
		{
			MSG objReturn = new MSG();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.FreeFormMessageText = elements[1];
				if (elements.Length > 2) objReturn.PrinterCarriageControlCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("PrinterCarriageControlCode", objReturn.PrinterCarriageControlCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.Number = Utils.ParseInt(elements[3]);
			}

			return objReturn;
		}
	}

	public static class NM1Manager
	{
		public static NM1 Parse(string segment)
		{
			NM1 objReturn = new NM1();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.EntityIdentifierCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("EntityIdentifierCode", objReturn.EntityIdentifierCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.EntityTypeQualifier = new IDENTIFIER(elements[2]);
				if (!Code.Validate("EntityTypeQualifier", objReturn.EntityTypeQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.NameLastOrOrganizationName = elements[3];
				if (elements.Length > 4) objReturn.NameFirst = elements[4];
				if (elements.Length > 5) objReturn.NameMiddle = elements[5];
				if (elements.Length > 6) objReturn.NamePrefix = elements[6];
				if (elements.Length > 7) objReturn.NameSuffix = elements[7];
				if (elements.Length > 8) objReturn.IdentificationCodeQualifier = new IDENTIFIER(elements[8]);
				if (!Code.Validate("IdentificationCodeQualifier", objReturn.IdentificationCodeQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.IdentificationCode = elements[9];
				if (elements.Length > 10) objReturn.EntityRelationshipCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("EntityRelationshipCode", objReturn.EntityRelationshipCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.EntityIdentifierCode2 = new IDENTIFIER(elements[11]);
				if (!Code.Validate("EntityIdentifierCode", objReturn.EntityIdentifierCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 12) objReturn.NameLastOrOrganizationName2 = elements[12];
			}

			return objReturn;
		}
	}

	public static class N2Manager
	{
		public static N2 Parse(string segment)
		{
			N2 objReturn = new N2();

			objReturn.Segment = segment;

			return objReturn;
		}
	}

	public static class N3Manager
	{
		public static N3 Parse(string segment)
		{
			N3 objReturn = new N3();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.AddressInformation1 = elements[1];
				if (elements.Length > 2) objReturn.AddressInformation2 = elements[2];
			}

			return objReturn;
		}
	}

	public static class N4Manager
	{
		public static N4 Parse(string segment)
		{
			N4 objReturn = new N4();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.CityName = elements[1];
				if (elements.Length > 2) objReturn.StateOrProvinceCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("StateOrProvinceCode", objReturn.StateOrProvinceCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.PostalCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("PostalCode", objReturn.PostalCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.CountryCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("CountryCode", objReturn.CountryCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.LocationQualifier = new IDENTIFIER(elements[5]);
				if (!Code.Validate("LocationQualifier", objReturn.LocationQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.LocationIdentifier = elements[6];
				if (elements.Length > 7) objReturn.CountrySubdivisionCode = new IDENTIFIER(elements[7]);
				if (!Code.Validate("CountrySubdivisionCode", objReturn.CountrySubdivisionCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class PERManager
	{
		public static PER Parse(string segment)
		{
			PER objReturn = new PER();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ContactFunctionCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("ContactFunctionCode", objReturn.ContactFunctionCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.Name = elements[2];
				if (elements.Length > 3) objReturn.CommunicationNumberQualifier1 = new IDENTIFIER(elements[3]);
				if (!Code.Validate("CommunicationNumberQualifier", objReturn.CommunicationNumberQualifier1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.CommunicationNumber1 = elements[4];
				if (elements.Length > 5) objReturn.CommunicationNumberQualifier2 = new IDENTIFIER(elements[5]);
				if (!Code.Validate("CommunicationNumberQualifier", objReturn.CommunicationNumberQualifier2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.CommunicationNumber2 = elements[6];
				if (elements.Length > 7) objReturn.CommunicationNumberQualifier3 = new IDENTIFIER(elements[7]);
				if (!Code.Validate("CommunicationNumberQualifier", objReturn.CommunicationNumberQualifier3))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.CommunicationNumber3 = elements[8];
				if (elements.Length > 9) objReturn.ContactInquiryReference = elements[9];
			}

			return objReturn;
		}
	}

	public static class PRVManager
	{
		public static PRV Parse(string segment)
		{
			PRV objReturn = new PRV();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.ProviderCode = new IDENTIFIER(elements[1]);
				if (!Code.Validate("ProviderCode", objReturn.ProviderCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.ReferenceIdentificationQualifier = new IDENTIFIER(elements[2]);
				if (!Code.Validate("ReferenceIdentificationQualifier", objReturn.ReferenceIdentificationQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.ReferenceIdentification = elements[3];
				if (elements.Length > 4) objReturn.StateOrProvinceCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("StateOrProvinceCode", objReturn.StateOrProvinceCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.ProviderSpecialtyInformation = new PROVIDER_SPECIALTY_INFORMATION(elements[5]);
				if (elements.Length > 6) objReturn.ProviderOrganizationCode = new IDENTIFIER(elements[6]);
				if (!Code.Validate("ProviderOrganizationCode", objReturn.ProviderOrganizationCode))
				{                  //TODO: Add AAA Error 
				}
			}

			return objReturn;
		}
	}

	public static class DMGManager
	{
		public static DMG Parse(string segment)
		{
			DMG objReturn = new DMG();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.DateTimePeriodFormatQualifier = new IDENTIFIER(elements[1]);
				if (!Code.Validate("DateTimePeriodFormatQualifier", objReturn.DateTimePeriodFormatQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.DateTimePeriod = elements[2];
				if (elements.Length > 3) objReturn.GenderCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("GenderCode", objReturn.GenderCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.MaritalStatusCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("MaritalStatusCode", objReturn.MaritalStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.CompositeRaceOrEthnicityInformation = new COMPOSITE_RACE_OR_ETHNICITY_INFORMATION(elements[5]);
				if (elements.Length > 6) objReturn.CitizenshipStatusCode = new IDENTIFIER(elements[6]);
				if (!Code.Validate("CitizenshipStatusCode", objReturn.CitizenshipStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 7) objReturn.CountryCode = new IDENTIFIER(elements[7]);
				if (!Code.Validate("CountryCode", objReturn.CountryCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.BasisOfVerificationCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("BasisOfVerificationCode", objReturn.BasisOfVerificationCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.Quantity = Utils.ParseDecimal(elements[9]);
				if (elements.Length > 10) objReturn.CodeListQualifierCode = new IDENTIFIER(elements[10]);
				if (!Code.Validate("CodeListQualifierCode", objReturn.CodeListQualifierCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.IndustryCode = elements[11];
			}

			return objReturn;
		}
	}

	public static class INSManager
	{
		public static INS Parse(string segment)
		{
			INS objReturn = new INS();

			string[] elements = segment.Split(Settings.Data_Element_Separator);
			if (elements != null)
			{
				if (elements.Length > 1) objReturn.YesNoConditionOrResponseCode1 = new IDENTIFIER(elements[1]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode1))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 2) objReturn.IndividualRelationshipCode = new IDENTIFIER(elements[2]);
				if (!Code.Validate("IndividualRelationshipCode", objReturn.IndividualRelationshipCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 3) objReturn.MaintenanceTypeCode = new IDENTIFIER(elements[3]);
				if (!Code.Validate("MaintenanceTypeCode", objReturn.MaintenanceTypeCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 4) objReturn.MaintenanceReasonCode = new IDENTIFIER(elements[4]);
				if (!Code.Validate("MaintenanceReasonCode", objReturn.MaintenanceReasonCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 5) objReturn.BenefitStatusCode = new IDENTIFIER(elements[5]);
				if (!Code.Validate("BenefitStatusCode", objReturn.BenefitStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 6) objReturn.MedicareStatusCode = new MEDICARE_STATUS_CODE(elements[6]);
				if (elements.Length > 7) objReturn.ConsolidatedOmnibusBudgetReconciliationActQualifying = new IDENTIFIER(elements[7]);
				if (!Code.Validate("ConsolidatedOmnibusBudgetReconciliationActQualifying", objReturn.ConsolidatedOmnibusBudgetReconciliationActQualifying))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 8) objReturn.EmploymentStatusCode = new IDENTIFIER(elements[8]);
				if (!Code.Validate("EmploymentStatusCode", objReturn.EmploymentStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 9) objReturn.StudentStatusCode = new IDENTIFIER(elements[9]);
				if (!Code.Validate("StudentStatusCode", objReturn.StudentStatusCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 10) objReturn.YesNoConditionOrResponseCode2 = new IDENTIFIER(elements[10]);
				if (!Code.Validate("YesNoConditionOrResponseCode", objReturn.YesNoConditionOrResponseCode2))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 11) objReturn.DateTimePeriodFormatQualifier = new IDENTIFIER(elements[11]);
				if (!Code.Validate("DateTimePeriodFormatQualifier", objReturn.DateTimePeriodFormatQualifier))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 12) objReturn.DateTimePeriod = elements[12];
				if (elements.Length > 13) objReturn.ConfidentialityCode = new IDENTIFIER(elements[13]);
				if (!Code.Validate("ConfidentialityCode", objReturn.ConfidentialityCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 14) objReturn.CityName = elements[14];
				if (elements.Length > 15) objReturn.StateOrProvinceCode = new IDENTIFIER(elements[15]);
				if (!Code.Validate("StateOrProvinceCode", objReturn.StateOrProvinceCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 16) objReturn.ContryCode = new IDENTIFIER(elements[16]);
				if (!Code.Validate("ContryCode", objReturn.ContryCode))
				{                  //TODO: Add AAA Error 
				}
				if (elements.Length > 17) objReturn.Number = Utils.ParseInt(elements[17]);
			}

			return objReturn;
		}
	}

}
