﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BO.X12._278;
using X215M = X12Engine.BLL.X12._278._005010X215;
using X215 = X12Engine.BO.X12._278._005010X215;
using X217M = X12Engine.BLL.X12._278._005010X217;
using X217 = X12Engine.BO.X12._278._005010X217;

namespace X12Engine.BLL.X12._278
{
   public static class MessageBaseManager
   {
      public static MessageBase Parse(string messageGroup)
      {
         MessageBase objReturn = null;

         Settings.Reset();  //Reset settings to default in case this message group doesn't have an ISA segment or separator values in the ISA segment

         string transactionSet = string.Empty;
         string implementationConventionReference = string.Empty;

         string[] segments = messageGroup.Split(Settings.Segment_Terminator);
         if (segments != null)
         {
            objReturn = new MessageBase();

            foreach (string segment in segments)
            {
               if (string.IsNullOrEmpty(segment)) continue;

               string[] elements = segment.Split(Settings.Data_Element_Separator);
               if (elements != null)
               {
                  string segmentId = elements[0].ToUpper();
                  switch (segmentId)
                  {
                     case "ISA":  // Interchange Control Header
                        objReturn.InterchangeControlHeader = ISAManager.Parse(segment);
                        break;

                     case "GS":   // FUnctional Group Header
                        objReturn.FunctionalGroupHeader = GSManager.Parse(segment);
                        break;

                     case "GE":   //  FUnctional Group Trailer
                        objReturn.FunctionalGroupTrailer = GEManager.Parse(segment);
                        break;

                     case "IEA":  // Interchange Control Trailer
								objReturn.InterchangeControlTrailer = IEAManager.Parse(segment);
                        break;

                     default:

                        if (segmentId == "ST")  // Transaction Set Header
                        {
                           transactionSet = segment + Settings.Segment_Terminator;
                           if (elements.Length > 3) implementationConventionReference = elements[3].ToUpper();
                        }
                        else if (segmentId == "SE")  // Transaction Set Trailer
								{
                           transactionSet += segment + Settings.Segment_Terminator;

                           if (implementationConventionReference == "005010X215")
                           {
                              X215.Message message = X215M.MessageManager.Parse(transactionSet);

                              if (objReturn.X215List == null) objReturn.X215List = new X215.MessageGroup();
                              objReturn.X215List.Add(message);
                           }
                           else if (implementationConventionReference == "005010X217")
                           {
                              X217.Message message = X217M.MessageManager.Parse(transactionSet);

                              if (objReturn.X217List == null) objReturn.X217List = new X217.MessageGroup();
                              objReturn.X217List.Add(message);
                           }
                        }
                        else
                        {
                           transactionSet += segment + Settings.Segment_Terminator;
                        }

                        break;
                  }
               }
            }
         }

         return objReturn;
      }
   }
}
