﻿	using System;
	using System.Collections.Generic;
	using System.Linq;
	using System.Text;

	namespace X12Engine.BO.X12._278._005010X215
	{
	[Serializable()]
	public class MessageGroup
	{
		#region Fields

		private List<Message> _messageList;

		#endregion

		#region Constructors

		public MessageGroup() { }

		#endregion

		#region Properties

		public List<Message> MessageList
		{
			get { return _messageList; }
			set { _messageList = value; }
		}

		#endregion

		#region Methods

		public void Add(Message message)
		{
			if (_messageList == null) _messageList = new List<Message>();
			_messageList.Add(message);
		}

		public bool ValidateMessage()
		{
			bool validMessage = true;

			if (_messageList == null)
			{
					return false;
			}
			else
			{
					foreach (Message m in _messageList)
					{
						//FS-278-3.0-001 SD-3.0-001
						if (m.TransactionSetHeader == null) { validMessage = false; }

						//FS-278-3.0-004 SD-3.0-003
						if (m.TransactionSetTrailer == null) { validMessage = false; }

						//FS-278-3.0-003 SD-3.0-002 //FS-278-3.0-006 SD-3.0-004
						if (m.TransactionSetHeader != null && m.TransactionSetTrailer != null)
						{
							if (m.TransactionSetHeader.TransactionSetControlNumber != m.TransactionSetTrailer.TransactionSetControlNumber)
							{
									validMessage = false;
							}
						}

						//FS-278-3.0-007 SD-3.0-005
						if (m.BeginningOfHierarchicalTransaction == null) { validMessage = false; }
						//FS-278-3.0-027 SD-3.0-238
						else if (m.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode == null
							|| m.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode.Code != "13")
						{
							validMessage = false;
						}

						//FS-278-3.0-049 SD-3.0-008
						if (m.UmoDetail == null) { validMessage = false; }
						//FS-278-3.0-060 SD-3.0-020
						else if (m.UmoDetail.UMOName == null || m.UmoDetail.UMOName.IndividualOrOrganizationName == null)
						{
							validMessage = false;
						}
						//FS-278-3.0-028 SD-3.0-239 //FS-278-3.0-029 SD-3.0-240 //FS-278-3.0-030 SD-3.0-241
						else if (m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode == null
							|| m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "1P"
							|| m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "2B"
							|| m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "36")
						{
							validMessage = false;
						}
						//FS-278-3.0-061-01
						else if (m.UmoDetail.UMOName.IndividualOrOrganizationName.IdentificationCode != "12115") { validMessage = false; }

						//FS-278-3.0-066 SD-3.0-030
						if (m.RequesterDetail == null) { validMessage = false; }
						//FS-278-3.0-072 SD-3.0-035
						else if (m.RequesterDetail.RequesterName == null) { validMessage = false; }

						//FS-278-3.0-085 SD-3.0-054
						if (m.SubscriberDetail == null) { validMessage = false; }
						//FS-278-3.0-092 SD-3.0-061
						else
						{
							bool foundRequiredElement = false;

							foreach (SubscriberDetail sd in m.SubscriberDetail)
							{
									if (sd.SubscriberName != null) foundRequiredElement = true;
							}

							if (!foundRequiredElement) validMessage = false;
						}

						//FS-278-3.0-108 SD-3.0-079 //FS-278-3.0-116 SD-3.0-085
						if (m.DependentDetail != null)
						{
							bool foundRequiredElement = false;

							foreach (DependentDetail dd in m.DependentDetail)
							{
									if (dd.DependentName != null) foundRequiredElement = true;
							}

							if (!foundRequiredElement) validMessage = false;
						}

						//FS-278-3.0-138 SD-3.0-107
						if (m.PatientEventDetail == null) { validMessage = false; }
						else
						{
							bool foundRequiredElement = false;

							foreach (PatientEventDetail ped in m.PatientEventDetail)
							{
									//FS-278-3.0-140 SD-3.0-109
									if (ped.HealthCareServicesReviewInformation != null)
									{
										foundRequiredElement = true;

										//FS-278-3.0-031 SD-3.0-242 //FS-278-3.0-033 SD-3.0-244
										if (ped.HealthCareServicesReviewInformation.RequestCategoryCode == null
											|| ped.HealthCareServicesReviewInformation.RequestCategoryCode.Code == "IN"
											|| ped.HealthCareServicesReviewInformation.RequestCategoryCode.Code == "SC")
										{
											validMessage = false;
										}

										//FS-278-3.0-034 SD-3.0-245 //FS-278-3.0-035 SD-3.0-246 //FS-278-3.0-036 SD-3.0-247 //FS-278-3.0-037 SD-3.0-248 
										//FS-278-3.0-038 SD-3.0-249 //FS-278-3.0-039 SD-3.0-250 //FS-278-3.0-040 SD-3.0-251
										if (ped.HealthCareServicesReviewInformation.CertificationTypeCode == null
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "1"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "2"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "3"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "4"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "5"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "N"
											|| ped.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "R")
										{
											validMessage = false;
										}
									}
							}

							if (!foundRequiredElement) validMessage = false;
						}

						if (m.ServiceDetail != null)
						{
							foreach (ServiceDetail sd in m.ServiceDetail)
							{
								if (sd.HealthCareServicesReviewInformation == null) { validMessage = false; }
								else
								{
									//FS-278-3.0-041 SD-3.0-252
									if (sd.HealthCareServicesReviewInformation.RequestCategoryCode == null
										|| sd.HealthCareServicesReviewInformation.RequestCategoryCode.Code != "HS")
									{
										validMessage = false;
									}

									//FS-278-3.0-042 SD-3.0-253 //FS-278-3.0-043 SD-3.0-254 //FS-278-3.0-044 SD-3.0-255
									//FS-278-3.0-045 SD-3.0-256 //FS-278-3.0-046 SD-3.0-257 //FS-278-3.0-047 SD-3.0-258
									if (sd.HealthCareServicesReviewInformation.CertificationTypeCode == null
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "1"
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "2"
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "3"
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "4"
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "N"
										|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "R")
									{
										validMessage = false;
									}
								}
							}
						}
					}
			}

			return validMessage;
		}

		public string GetUAK()
		{
			string UAK = string.Empty;

			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.PatientEventDetail != null)
					{
						foreach (PatientEventDetail ped in m.PatientEventDetail)
						{
							//TODO: Lookup by (TRN) Patient Event Tracking Number, 
							//up to 9 TRNs allowed, use last one?
							//if (ped.PatientEventTraceNumber != null)
							//{
							//    foreach (TRN t in ped.PatientEventTraceNumber)
							//    {
							//        if (!string.IsNullOrEmpty(t.ReferenceIdentification1))
							//        {
							//            UAK = t.ReferenceIdentification1;
							//            return UAK; //return first authorization record found???
							//        }
							//    }
							//}

							//TODO: Which REF value should we use for alt UAK lookup, 
							//up to 9 REFs allowed, two specific ones and up to 7 other generic ones?
							if (string.IsNullOrEmpty(UAK))
							{
								if (ped.PreviousReviewAuthorizationNumber != null)
								{
									UAK = ped.PreviousReviewAuthorizationNumber.ReferenceIdentification;
									return UAK; //return first authorization record found???
								}

								if (ped.PreviousReviewAdministrativeReferenceNumber != null)
								{
									UAK = ped.PreviousReviewAdministrativeReferenceNumber.ReferenceIdentification;
									return UAK; //return first authorization record found???
								}
							}
						}
					}
				}
			}

			return UAK;
		}

		public void GetPatientInfo(ref string lastName, ref string firstName, ref string ssn, ref DateTime? dateOfBirth)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.DependentDetail != null)
					{
						foreach (DependentDetail dd in m.DependentDetail)
						{
								if (dd.DependentName != null)
								{
									dd.DependentName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
									return;  //return first dependent record found???
								}
						}
					}

					if (m.SubscriberDetail != null)
					{
						foreach (SubscriberDetail sd in m.SubscriberDetail)
						{
								if (sd.SubscriberName != null)
								{
									sd.SubscriberName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
									return; //return first subscriber record found???
								}
						}
					}
				}
			}
		}

		public void GetVendorInfo(ref string vendorTin, ref string vendorName)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.RequesterDetail != null)
					{
						if (m.RequesterDetail.RequesterName != null)
						{
								foreach (EntityBase eb in m.RequesterDetail.RequesterName)
								{
									//TODO: How do we handle multiplerecords
									if (eb.IndividualOrOrganizationName != null)
										eb.IndividualOrOrganizationName.GetVendorInfo(ref vendorTin, ref vendorName);
								}
						}
					}
				}
			}
		}

		public void GetLookupDates(ref DateTime? issueDate, ref DateTime? fromDate, ref DateTime? toDate)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.PatientEventDetail != null)
					{
						foreach (PatientEventDetail ped in m.PatientEventDetail)
						{
								//TODO: How do we handle multiplerecords
								if (ped.EventDate != null) ped.EventDate.GetDates(ref fromDate, ref toDate);
								if (ped.CertificationIssueDate != null) ped.CertificationIssueDate.GetDate(ref issueDate);
						}
					}
				}
			}
		}

		public void UpdateUmoDetail(Int16 STATION_CODE, string STATION_NAME)
		{
			//FS-278-3.0-065-01

			if (_messageList != null)
			{
					foreach (Message m in _messageList)
					{
						if (m.UmoDetail != null && m.UmoDetail.UMOName != null && m.UmoDetail.UMOName.IndividualOrOrganizationName != null)
						{
							m.UmoDetail.UMOName.IndividualOrOrganizationName.NameLastOrOrganizationName = STATION_CODE.ToString() + "-" + STATION_NAME;
						}
					}
			}
		}

		public void UpdateRequesterDetail()
		{
			if (_messageList != null)
			{
					foreach (Message m in _messageList)
					{
						if (m.RequesterDetail != null)
						{
							if (m.RequesterDetail.RequesterName != null)
							{
								foreach (EntityBase eb in m.RequesterDetail.RequesterName)
								{
									//FS-278-3.0-079
									eb.AdministrativeCommunicationsContact = null;
									//FS-278-3.0-080
									eb.ProviderInformation = null;
								}
							}
						}
					}
			}
		}

		public void UpdateSubscriberDetail()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{

				}
			}
		}

		public void UpdateDependentDetail()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{

				}
			}
		}

		public void UpdatePatientEventDetail(string UAK, DateTime dateFrom, DateTime dateTo, string services)
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					int parentIdNumber = GetPatientEventParentId(m);
					int hierarchicalIdNumber = parentIdNumber + 1;
					List<TRN> trnList = GetPatientEventTrnList(m);
					List<UM> umList = GetPatientEventUmList(m);

					PatientEventDetail patientEventDetail = new PatientEventDetail();

					//FS-278-3.0-157 SD-3.0-166
					patientEventDetail.PatientEventLevel = new HL(hierarchicalIdNumber.ToString(), parentIdNumber.ToString(), new IDENTIFIER("EV"), new IDENTIFIER("0"));
                    
					//FS-278-3.0-158 SD-3.0-167
					if (trnList != null && trnList.Count > 0) patientEventDetail.PatientEventTraceNumber = trnList;
                    
					//FS-278-3.0-159 SD-3.0-168
					//TODO: How do we handle the possibility of multiple UM's with only one PatientEventDetail response
					if (umList != null && umList.Count > 0) patientEventDetail.HealthCareServicesReviewInformation = umList[0];

					//FS-278-3.0-160 SD-3.0-170
					//TODO: Which REF?  Previous Review Authorization Number or Previous Review Administrative Reference Number? Up to 9 REFs are allowed by the standard.
					patientEventDetail.PreviousReviewAuthorizationNumber = new REF("BB", UAK);

					//FS-278-3.0-161 SD-3.0-172 //FS-278-3.0-161-01 //FS-278-3.0-161-02 //FS-278-3.0-161-03 //FS-278-3.0-161-04 //FS-278-3.0-161-05
					patientEventDetail.EventDate = new DTP("AAH", dateFrom, dateTo);

					//FS-278-3.0-162 SD-3.0-173
					if (!string.IsNullOrEmpty(services)) patientEventDetail.MessageText = new MSG(services);

					m.PatientEventDetail = new List<PatientEventDetail>();
					m.PatientEventDetail.Add(patientEventDetail);
				}
			}
		}

		public void UpdateServiceDetail()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					//FS-278-3.0-174 SD-3.0-236 //FS-278-3.0-175 SD-3.0-237
					m.ServiceDetail = null;
				}
			}
		}

		public void UpdateSegmentIds()
		{
			//FS-278-3.0-054 //FS-278-3.0-070

			int idNumber = 0;

			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					if (m.UmoDetail != null && m.UmoDetail.UMOLevel != null)
					{
						idNumber++;

						m.UmoDetail.UMOLevel.HierarchicalIdNumber = idNumber.ToString();
					}

					if (m.RequesterDetail != null && m.RequesterDetail.RequesterLevel != null)
					{
						idNumber++;

						m.RequesterDetail.RequesterLevel.HierarchicalIdNumber = idNumber.ToString();
					}

					if (m.SubscriberDetail != null)
					{
						foreach (SubscriberDetail sd in m.SubscriberDetail)
						{
							idNumber++;

							if (sd.SubscriberLevel != null) sd.SubscriberLevel.HierarchicalIdNumber = idNumber.ToString();
						}
					}

					if (m.DependentDetail != null)
					{
						foreach (DependentDetail dd in m.DependentDetail)
						{
							idNumber++;

							if (dd.DependentLevel != null) dd.DependentLevel.HierarchicalIdNumber = idNumber.ToString();
						}
					}

					if (m.PatientEventDetail != null)
					{
						foreach (PatientEventDetail ped in m.PatientEventDetail)
						{
							idNumber++;

							if (ped.PatientEventLevel != null) ped.PatientEventLevel.HierarchicalIdNumber = idNumber.ToString();
						}
					}

					if (m.ServiceDetail != null)
					{
						foreach (ServiceDetail sd in m.ServiceDetail)
						{
							idNumber++;

							if (sd.ServiceLevel != null) sd.ServiceLevel.HierarchicalIdNumber = idNumber.ToString();
						}
					}
				}
			}
		}

		public void UpdateSegmentCount()
		{
			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					int segmentCount = Utils.CountSegments(m.ToString());

					if (m.TransactionSetTrailer == null) m.TransactionSetTrailer = new SE();

					if (m.TransactionSetHeader != null)
						m.TransactionSetTrailer.TransactionSetControlNumber = m.TransactionSetHeader.TransactionSetControlNumber;
					m.TransactionSetTrailer.NumberOfIncludedSegments = segmentCount;
				}
			}
		}

		public override string ToString()
		{
			string strReturn = string.Empty;

			if (_messageList != null)
			{
				foreach (Message m in _messageList)
				{
					strReturn += m.ToString();
				}
			}

			return strReturn;
		}

		#endregion

		#region Helpers

		private int GetPatientEventParentId(Message m)
		{
			int tempId = 0;
			int parentIdNumber = 0;

			if (m.DependentDetail != null)
			{
				foreach (DependentDetail dd in m.DependentDetail)
				{
					if (dd.DependentLevel != null)
					{
						if (int.TryParse(dd.DependentLevel.HierarchicalIdNumber, out tempId))
						{
								if (tempId > parentIdNumber) parentIdNumber = tempId;
						}
					}
				}
			}

			if (parentIdNumber > 0) return parentIdNumber;

			if (m.SubscriberDetail != null)
			{
					foreach (SubscriberDetail sd in m.SubscriberDetail)
					{
						if (sd.SubscriberLevel != null)
						{
							if (int.TryParse(sd.SubscriberLevel.HierarchicalIdNumber, out tempId))
							{
									if (tempId > parentIdNumber) parentIdNumber = tempId;
							}
						}
					}
			}

			return parentIdNumber;
		}

		private List<TRN> GetPatientEventTrnList(Message m)
		{
			List<TRN> trnList = null;

			if (m.PatientEventDetail != null)
			{
				trnList = new List<TRN>();

				foreach (PatientEventDetail ped in m.PatientEventDetail)
				{
					if (ped.PatientEventTraceNumber != null)
					{
						foreach (TRN t in ped.PatientEventTraceNumber)
						{
								TRN trn = t.Clone();
								trnList.Add(trn);
						}
					}
				}
			}

			return trnList;
		}

		private List<UM> GetPatientEventUmList(Message m)
		{
			List<UM> umList = null;

			if (m.PatientEventDetail != null)
			{
				umList = new List<UM>();

				foreach (PatientEventDetail ped in m.PatientEventDetail)
				{
					if (ped.HealthCareServicesReviewInformation != null)
					{
						UM um = ped.HealthCareServicesReviewInformation.Clone();
						umList.Add(um);
					}
				}
			}

			return umList;
		}

		#endregion
	}
}
