﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using X12Engine.BLL.X12._278;

namespace X12Engine.BO.X12._278._005010X217
{
   [Serializable()]
   public class MessageGroup
   { 

      private List<Message> _messageList;

      public MessageGroup() { }

      public List<Message> MessageList
      {
         get { return _messageList; }
         set { _messageList = value; }
      }

      public void Add(Message message)
      {
         if (_messageList == null) _messageList = new List<Message>();
         _messageList.Add(message);
      }

      public bool ValidateMessage()
      {
         Utils.LogEvent("X217.ValidateMessage()");
         bool validMessage = true;

			foreach (Message m in _messageList)
			{

				//****************************************************************************  Validate Transaction controls 
				//FS-278-3.0-001 
				if (m.TransactionSetHeader == null)
				{
					validMessage = false;
				}

				//FS-278-3.0-004 
				if (m.TransactionSetTrailer == null)
				{
					validMessage = false;
				}

				//FS-278-3.0-003    //FS-278-3.0-006 
				if (m.TransactionSetHeader != null && m.TransactionSetTrailer != null)
				{
					if (m.TransactionSetHeader.TransactionSetControlNumber != m.TransactionSetTrailer.TransactionSetControlNumber)
					{
						validMessage = false;
					}
				}

				//FS-278-3.0-007 SD-3.0-005  //FS-278-3.0-027 SD-3.0-238
				if (m.BeginningOfHierarchicalTransaction == null)
				{
					validMessage = false;
				} else if (m.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode == null || m.BeginningOfHierarchicalTransaction.TransactionSetPurposeCode.Code != "13")  
				{
					validMessage = false;
				}




				//****************************************************************************  UmoDetail (HL1)
				//FS-278-3.0-049 SD-3.0-008  //FS-278-3.0-060 SD-3.0-020  //FS-278-3.0-028 SD-3.0-239 //FS-278-3.0-029 SD-3.0-240 //FS-278-3.0-030 SD-3.0-241  //FS-278-3.0-061-01
				bool validUmoDetail = true;
				if (m.UmoDetail == null)
				{
					AAAManager.AddUmoDetailError("N", "AA", "C", "UmoDetail Error");
					validUmoDetail = false;
				} else if (m.UmoDetail.UMOName == null || m.UmoDetail.UMOName.IndividualOrOrganizationName == null)
				{
					AAAManager.AddUmoDetailError("N", "AA", "C", "UmoDetail Error");
					validUmoDetail = false;
				} else if (m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode == null || m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "1P" || m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "2B" || m.UmoDetail.UMOName.IndividualOrOrganizationName.EntityIdentifierCode.Code == "36")
				{
					AAAManager.AddUmoDetailError("N", "AA", "C", "UmoDetail Error");
					validUmoDetail = false;
				} else if (m.UmoDetail.UMOName.IndividualOrOrganizationName.IdentificationCode != "12115")
				{
					AAAManager.AddUmoDetailError("N", "AA", "C", "UmoDetail Error");
					validUmoDetail = false;
				}

				if (!validUmoDetail) return false;




				//****************************************************************************  RequesterDetail (HL2)
				//FS-278-3.0-066 SD-3.0-030  //FS-278-3.0-072 SD-3.0-035
				bool validRequesterDetail = true;
				if (m.RequesterDetail == null)
				{
					AAAManager.AddRequesterDetailError("N", "AA", "C", "RequesterDetail Error");
					validRequesterDetail = false;
				} else if (m.RequesterDetail.RequesterName == null)
				{
					AAAManager.AddRequesterDetailError("N", "AA", "C", "RequesterDetail Error");
					validRequesterDetail = false;
				}

				if (!validRequesterDetail) return false;




				//****************************************************************************  SubscriberDetail (HL3)  
				//FS-278-3.0-085 SD-3.0-054   //FS-278-3.0-092 SD-3.0-061   //FS-278-3.0-088 SD-3.0-057

				bool validSubscriberDetail = true;
				if (m.SubscriberDetail == null)
				{
					AAAManager.AddSubscriberDetailError("N", "AA", "C", "AddSubscriberDetailError Error");
					validSubscriberDetail = false;
				}
				else
				{
					bool foundRequiredElement = false;

					foreach (SubscriberDetail sd in m.SubscriberDetail)
					{
						if (sd.SubscriberName != null) foundRequiredElement = true;

						if (sd.Other != null)
						{
							if (sd.Other.Trace != null)
							{

								sd.Other.Trace = null;
							}
						}
					}

					if (!foundRequiredElement)
					{
						AAAManager.AddSubscriberDetailError("N", "AA", "C", "AddSubscriberDetailError Error");
						validSubscriberDetail = false;
					}

				}

				if (!validSubscriberDetail) return false;








				//****************************************************************************  DependentDetail (NA)  
				//FS-278-3.0-108 SD-3.0-079   //FS-278-3.0-116 SD-3.0-085    //FS-278-3.0-109 SD-3.0-080
				bool validDependentDetail = true;
				if (m.DependentDetail != null)
				{
					bool foundRequiredElement = false;

					foreach (DependentDetail dd in m.DependentDetail)
					{
						if (dd.DependentName != null) foundRequiredElement = true;

						if (dd.Other != null)
						{
							
							if (dd.Other.Trace != null)
							{
								dd.Other.Trace = null;
							}
						}
					}

					if (!foundRequiredElement)
					{
						AAAManager.AddDependentDetailError("N", "AA", "C", "AddDependentDetailError Error");
						validDependentDetail = false;
					}
				}
				if (!validDependentDetail) return false;







				//****************************************************************************  PatientEventDetail (HL4)  
				//FS-278-3.0-138 SD-3.0-107
				bool validPatientEventDetail = true;

				if (m.PatientEventDetail == null)
				{
					AAAManager.AddPatientEventDetailError("N", "AA", "C", "AddPatientEventDetailError Error");
					validPatientEventDetail = false;
				}
				else
				{
					bool foundRequiredElement = false;

					foreach (PatientEventDetail patientEventDetail in m.PatientEventDetail)
					{



						//EventDate
						if (patientEventDetail.EventDate.DateTimePeriod.Length == 0)
						{
							AAAManager.AddPatientEventDetailError("N", "AA", "C", "AddPatientEventDetailError Error");
							validPatientEventDetail = false;
						}





						//FS-278-3.0-140 SD-3.0-109
						if (patientEventDetail.HealthCareServicesReviewInformation != null)
						{
							foundRequiredElement = true;

							//FS-278-3.0-031 SD-3.0-242 //FS-278-3.0-033 SD-3.0-244
							string requestCategoryCode = (patientEventDetail.HealthCareServicesReviewInformation.RequestCategoryCode != null) ? patientEventDetail.HealthCareServicesReviewInformation.RequestCategoryCode.Code : "";
							if (patientEventDetail.HealthCareServicesReviewInformation.RequestCategoryCode == null || requestCategoryCode == "IN" || requestCategoryCode == "SC")
							{
								AAAManager.AddPatientEventDetailError("N", "AA", "C", "AddPatientEventDetailError Error");

								validPatientEventDetail = false;
							}

							//FS-278-3.0-034 SD-3.0-245 //FS-278-3.0-035 SD-3.0-246 //FS-278-3.0-036 SD-3.0-247 //FS-278-3.0-037 SD-3.0-248 
							//FS-278-3.0-038 SD-3.0-249 //FS-278-3.0-039 SD-3.0-250 //FS-278-3.0-040 SD-3.0-251
							if (patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode == null
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "1"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "2"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "3"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "4"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "5"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "N"
								|| patientEventDetail.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "R")
							{
								AAAManager.AddPatientEventDetailError("N", "AA", "C", "AddPatientEventDetailError Error");
								validPatientEventDetail = false;
							}
						}
					}

					if (!foundRequiredElement)
					{
						AAAManager.AddPatientEventDetailError("N", "AA", "C", "AddPatientEventDetailError Error");
						validPatientEventDetail = false;
					}
				}

				//xxxif (!validPatientEventDetail) return false;










				//****************************************************************************  ServiceDetail (NA) 
				bool validServiceDetail = true;
				if (m.ServiceDetail != null)
				{
					foreach (ServiceDetail sd in m.ServiceDetail)
					{
						if (sd.HealthCareServicesReviewInformation == null) { validMessage = false; }
						else
						{
							//FS-278-3.0-041 SD-3.0-252
							if (sd.HealthCareServicesReviewInformation.RequestCategoryCode == null || sd.HealthCareServicesReviewInformation.RequestCategoryCode.Code != "HS")
							{
								AAAManager.AddServiceDetailError("N", "AA", "C", "AddServiceDetailError Error");
								validServiceDetail = false;
							}

							//FS-278-3.0-042 SD-3.0-253 //FS-278-3.0-043 SD-3.0-254 //FS-278-3.0-044 SD-3.0-255
							//FS-278-3.0-045 SD-3.0-256 //FS-278-3.0-046 SD-3.0-257 //FS-278-3.0-047 SD-3.0-258
							if (sd.HealthCareServicesReviewInformation.CertificationTypeCode == null
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "1"
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "2"
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "3"
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "4"
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "N"
								|| sd.HealthCareServicesReviewInformation.CertificationTypeCode.Code == "R")
							{
								AAAManager.AddServiceDetailError("N", "AA", "C", "AddServiceDetailError Error");
								validServiceDetail = false;
							}
						}
					}
				}
				if (!validServiceDetail) return false;

			}
			

			return validMessage;
      }

      public void FormatError()
      {
         foreach (Message m in _messageList)
         {
				if (AAAManager.UmoDetailAAA.Count > 0)
				{
					m.UmoDetail.RequestValidation = AAAManager.UmoDetailAAA;
            }

				if (AAAManager.RequesterDetailAAA.Count > 0)
				{
					m.RequesterDetail.RequestValidation = AAAManager.RequesterDetailAAA;
				}


				if (AAAManager.SubscriberDetailAAA.Count > 0)
				{
					foreach (SubscriberDetail subscriberDetail in m.SubscriberDetail)
					{
						subscriberDetail.RequestValidation = AAAManager.SubscriberDetailAAA;
					}
				}

				if (AAAManager.DependentDetailAAA.Count > 0)
				{
					foreach (DependentDetail dependentDetail in m.DependentDetail)
					{
						dependentDetail.RequestValidation = AAAManager.DependentDetailAAA;
					}
				}


				if (AAAManager.PatientEventDetailAAA.Count > 0)
				{
					foreach (PatientEventDetail patientEventDetail in m.PatientEventDetail)
					{
						patientEventDetail.RequestValidation = AAAManager.PatientEventDetailAAA;
					}
				}

				if (AAAManager.SubscriberDetailAAA.Count > 0)
				{
					foreach (SubscriberDetail subscriberDetail in m.SubscriberDetail)
					{
						subscriberDetail.RequestValidation = AAAManager.SubscriberDetailAAA;
					}
				}

			}
		}




      public string GetUAK()
      {
         string UAK = string.Empty;

         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {
               if (m.PatientEventDetail != null)
               {
                  foreach (PatientEventDetail ped in m.PatientEventDetail)
                  {
                     //TODO: Lookup by (TRN) Patient Event Tracking Number, 
                     //up to 9 TRNs allowed, use last one?
                     //if (ped.PatientEventTraceNumber != null)
                     //{
                     //    foreach (TRN t in ped.PatientEventTraceNumber)
                     //    {
                     //        if (!string.IsNullOrEmpty(t.ReferenceIdentification1))
                     //        {
                     //            UAK = t.ReferenceIdentification1;
                     //            return UAK; //return first authorization record found???
                     //        }
                     //    }
                     //}

                     //TODO: Which REF value should we use for alt UAK lookup, 
                     //up to 9 REFs allowed, two specific ones and up to 7 other generic ones?
                     if (string.IsNullOrEmpty(UAK))
                     {
                        if (ped.PreviousReviewAuthorizationNumber != null)
                        {
                           UAK = ped.PreviousReviewAuthorizationNumber.ReferenceIdentification;
                           return UAK; //return first authorization record found???
                        }

                        if (ped.PreviousReviewAdministrativeReferenceNumber != null)
                        {
                           UAK = ped.PreviousReviewAdministrativeReferenceNumber.ReferenceIdentification;
                           return UAK; //return first authorization record found???
                        }
                     }
                  }
               }
            }
         }

         return UAK;
      }

      public void GetPatientInfo(ref string lastName, ref string firstName, ref string ssn, ref DateTime? dateOfBirth)
      {
         Utils.LogEvent("GetPatientInfo217()");
         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {
               if (m.DependentDetail != null)
               {
                  foreach (DependentDetail dd in m.DependentDetail)
                  {
                     if (dd.DependentName != null)
                     {
                        dd.DependentName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
                        return;  //return first dependent record found???
                     }
                  }
               }

               if (m.SubscriberDetail != null)
               {
                  foreach (SubscriberDetail sd in m.SubscriberDetail)
                  {
                     if (sd.SubscriberName != null)
                     {

                        sd.SubscriberName.GetPatientInfo(ref lastName, ref firstName, ref ssn, ref dateOfBirth);
                        return; //return first subscriber record found???
                     }
                  }
               }
            }
         }
      }

      public void GetVendorInfo(ref string vendorTin, ref string vendorName)
      {
         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  if (m.RequesterDetail != null)
                  {
                     if (m.RequesterDetail.RequesterName != null)
                     {
                           foreach (EntityBase eb in m.RequesterDetail.RequesterName)
                           {
                              //TODO: How do we handle multiplerecords
                              if (eb.IndividualOrOrganizationName != null)
                                 eb.IndividualOrOrganizationName.GetVendorInfo(ref vendorTin, ref vendorName);
                           }
                     }
                  }
               }
         }
      }

      public void GetLookupDates(ref DateTime? issueDate, ref DateTime? fromDate, ref DateTime? toDate)
      {
         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {
               if (m.PatientEventDetail != null)
               {
                  foreach (PatientEventDetail ped in m.PatientEventDetail)
                  {
                     if (ped.EventDate != null) ped.EventDate.GetDates(ref fromDate, ref toDate);
                     if (ped.CertificationIssueDate != null) ped.CertificationIssueDate.GetDate(ref issueDate);
                  }
               }
            }
         }
      }

      public void UpdateUmoDetail(Int16 STATION_CODE, string STATION_NAME)
      {
         //FS-278-3.0-065-01

         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  if (m.UmoDetail != null && m.UmoDetail.UMOName != null && m.UmoDetail.UMOName.IndividualOrOrganizationName != null)
                  {
                     m.UmoDetail.UMOName.IndividualOrOrganizationName.NameLastOrOrganizationName = STATION_CODE.ToString() + "-" + STATION_NAME;
                  }
               }
         }
      }

      public void UpdateRequesterDetail()
      {
         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  if (m.RequesterDetail != null)
                  {
                     if (m.RequesterDetail.RequesterName != null)
                     {
                           foreach (EntityBase eb in m.RequesterDetail.RequesterName)
                           {
                              //FS-278-3.0-079
                              eb.AdministrativeCommunicationsContact = null;
                              //FS-278-3.0-080
                              eb.ProviderInformation = null;
                           }
                     }
                  }
               }
         }
      }

      public void UpdateSubscriberDetail()
      {
         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {

               }
         }
      }

      public void UpdateDependentDetail()
      {
         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {

            }
         }
      }

      public void UpdatePatientEventDetail(string UAK, DateTime dateFrom, DateTime dateTo, string services)
      {
         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {
               int parentIdNumber = GetPatientEventParentId(m);
               int hierarchicalIdNumber = parentIdNumber + 1;
               List<TRN> trnList = GetPatientEventTrnList(m);
               List<UM> umList = GetPatientEventUmList(m);

               PatientEventDetail patientEventDetail = new PatientEventDetail();

               //FS-278-3.0-157 SD-3.0-166
               patientEventDetail.PatientEventLevel = new HL(hierarchicalIdNumber.ToString(), parentIdNumber.ToString(), new IDENTIFIER("EV"), new IDENTIFIER("0"));

               //FS-278-3.0-158 SD-3.0-167
               if (trnList != null && trnList.Count > 0) patientEventDetail.PatientEventTraceNumber = trnList;

               //FS-278-3.0-159 SD-3.0-168
               //TODO: How do we handle the possibility of multiple UM's with only one PatientEventDetail response
               if (umList != null && umList.Count > 0) patientEventDetail.HealthCareServicesReviewInformation = umList[0];

               //FS-278-3.0-160 SD-3.0-170
               //TODO: Which REF?  Previous Review Authorization Number or Previous Review Administrative Reference Number? Up to 9 REFs are allowed by the standard.
               patientEventDetail.PreviousReviewAuthorizationNumber = new REF("BB", UAK);

               //FS-278-3.0-161 SD-3.0-172 //FS-278-3.0-161-01 //FS-278-3.0-161-02 //FS-278-3.0-161-03 //FS-278-3.0-161-04 //FS-278-3.0-161-05
               patientEventDetail.EventDate = new DTP("AAH", dateFrom, dateTo);

               //FS-278-3.0-162 SD-3.0-173
               if (!string.IsNullOrEmpty(services)) patientEventDetail.MessageText = new MSG(services);

               m.PatientEventDetail = new List<PatientEventDetail>();
               m.PatientEventDetail.Add(patientEventDetail);
            }
         }
      }



      public void UpdateServiceDetail()
      {
         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  //FS-278-3.0-174 SD-3.0-236 //FS-278-3.0-175 SD-3.0-237
                  m.ServiceDetail = null;
               }
         }
      }

      public void UpdateSegmentIds()
      {
         //FS-278-3.0-054 //FS-278-3.0-070

         int idNumber = 0;

         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  if (m.UmoDetail != null && m.UmoDetail.UMOLevel != null)
                  {
                     idNumber++;

                     m.UmoDetail.UMOLevel.HierarchicalIdNumber = idNumber.ToString();
                  }

                  if (m.RequesterDetail != null && m.RequesterDetail.RequesterLevel != null)
                  {
                     idNumber++;

                     m.RequesterDetail.RequesterLevel.HierarchicalIdNumber = idNumber.ToString();
                  }

                  if (m.SubscriberDetail != null)
                  {
                     foreach (SubscriberDetail sd in m.SubscriberDetail)
                     {
                           idNumber++;

                           if (sd.SubscriberLevel != null) sd.SubscriberLevel.HierarchicalIdNumber = idNumber.ToString();
                     }
                  }

                  if (m.DependentDetail != null)
                  {
                     foreach (DependentDetail dd in m.DependentDetail)
                     {
                           idNumber++;

                           if (dd.DependentLevel != null) dd.DependentLevel.HierarchicalIdNumber = idNumber.ToString();
                     }
                  }

                  if (m.PatientEventDetail != null)
                  {
                     foreach (PatientEventDetail ped in m.PatientEventDetail)
                     {
                           idNumber++;

                           if (ped.PatientEventLevel != null) ped.PatientEventLevel.HierarchicalIdNumber = idNumber.ToString();
                     }
                  }

                  if (m.ServiceDetail != null)
                  {
                     foreach (ServiceDetail sd in m.ServiceDetail)
                     {
                           idNumber++;

                           if (sd.ServiceLevel != null) sd.ServiceLevel.HierarchicalIdNumber = idNumber.ToString();
                     }
                  }
               }
         }
      }

      public void UpdateSegmentCount()
      {
         if (_messageList != null)
         {
               foreach (Message m in _messageList)
               {
                  int segmentCount = Utils.CountSegments(m.ToString());

                  if (m.TransactionSetTrailer == null) m.TransactionSetTrailer = new SE();

                  if (m.TransactionSetHeader != null)
                     m.TransactionSetTrailer.TransactionSetControlNumber = m.TransactionSetHeader.TransactionSetControlNumber;
                  m.TransactionSetTrailer.NumberOfIncludedSegments = segmentCount;
               }
         }
      }

      public override string ToString()
      {
         string strReturn = string.Empty;

         if (_messageList != null)
         {
            foreach (Message m in _messageList)
            {
               strReturn += m.ToString();
            }
         }

         return strReturn;
      }


      private int GetPatientEventParentId(Message m)
      {
         int tempId = 0;
         int parentIdNumber = 0;

         if (m.DependentDetail != null)
         {
               foreach (DependentDetail dd in m.DependentDetail)
               {
                  if (dd.DependentLevel != null)
                  {
                     if (int.TryParse(dd.DependentLevel.HierarchicalIdNumber, out tempId))
                     {
                           if (tempId > parentIdNumber) parentIdNumber = tempId;
                     }
                  }
               }
         }

         if (parentIdNumber > 0) return parentIdNumber;

         if (m.SubscriberDetail != null)
         {
               foreach (SubscriberDetail sd in m.SubscriberDetail)
               {
                  if (sd.SubscriberLevel != null)
                  {
                     if (int.TryParse(sd.SubscriberLevel.HierarchicalIdNumber, out tempId))
                     {
                           if (tempId > parentIdNumber) parentIdNumber = tempId;
                     }
                  }
               }
         }

         return parentIdNumber;
      }

      private List<TRN> GetPatientEventTrnList(Message m)
      {
         List<TRN> trnList = null;

         if (m.PatientEventDetail != null)
         {
               trnList = new List<TRN>();

               foreach (PatientEventDetail ped in m.PatientEventDetail)
               {
                  if (ped.PatientEventTraceNumber != null)
                  {
                     foreach (TRN t in ped.PatientEventTraceNumber)
                     {
                           TRN trn = t.Clone();
                           trnList.Add(trn);
                     }
                  }
               }
         }

         return trnList;
      }

      private List<UM> GetPatientEventUmList(Message m)
      {
         List<UM> umList = null;

         if (m.PatientEventDetail != null)
         {
               umList = new List<UM>();

               foreach (PatientEventDetail ped in m.PatientEventDetail)
               {
                  if (ped.HealthCareServicesReviewInformation != null)
                  {
                     UM um = ped.HealthCareServicesReviewInformation.Clone();
                     umList.Add(um);
                  }
               }
         }

         return umList;
      }

   }
}
