﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace X12Engine.BO.X12._278
{
    [Serializable()]
    public class DetailBase
    {
        #region Fields

        private HL _hierarchicalLevel;
        private List<TRN> _trace;
        private List<AAA> _requestValidation;
        private UM _healthCareServicesReviewInformation;
        private HCR _healthCareServicesReview;
        private List<REF> _referenceInformation;
        private List<DTP> _dateOrTimeOrPeriod;
        private HI _healthCareInformationCodes;
        private SV1 _professionalService;
        private SV2 _institutionalService;
        private SV3 _dentalService;
        private List<TOO> _toothIdentification;
        private HSD _healthCareServicesDelivery;
        private List<CRC> _conditionsIndicator;
        private CL1 _claimCodes;
        private CR1 _ambulanceCertification;
        private CR2 _chiropracticCertification;
        private CR4 _enteralOrParenteralTherapyCertification;
        private CR5 _oxygenTherapyCertification;
        private CR6 _homeHealthCareCertification;
        private CR7 _homeHealthTreatmentPlanCertification;
        private CR8 _pacemakerCertification;
        private List<PWK> _paperwork;
        private MSG _messageText;

        #endregion

        #region Constructors

        public DetailBase() { }

        #endregion

        #region Properties

        public HL HierarchicalLevel
        {
            get { return _hierarchicalLevel; }
            set { _hierarchicalLevel = value; }
        }

        public List<TRN> Trace
        {
            get { return _trace; }
            set { _trace = value; }
        }

        public List<AAA> RequestValidation
        {
            get { return _requestValidation; }
            set { _requestValidation = value; }
        }

        public UM HealthCareServicesReviewInformation
        {
            get { return _healthCareServicesReviewInformation; }
            set { _healthCareServicesReviewInformation = value; }
        }

        public HCR HealthCareServicesReview
        {
            get { return _healthCareServicesReview; }
            set { _healthCareServicesReview = value; }
        }

        public List<REF> ReferenceInformation
        {
            get { return _referenceInformation; }
            set { _referenceInformation = value; }
        }

        public List<DTP> DateOrTimeOrPeriod
        {
            get { return _dateOrTimeOrPeriod; }
            set { _dateOrTimeOrPeriod = value; }
        }

        public HI HealthCareInformationCodes
        {
            get { return _healthCareInformationCodes; }
            set { _healthCareInformationCodes = value; }
        }

        public SV1 ProfessionalService
        {
            get { return _professionalService; }
            set { _professionalService = value; }
        }

        public SV2 InstitutionalService
        {
            get { return _institutionalService; }
            set { _institutionalService = value; }
        }

        public SV3 DentalService
        {
            get { return _dentalService; }
            set { _dentalService = value; }
        }

        public List<TOO> ToothIdentification
        {
            get { return _toothIdentification; }
            set { _toothIdentification = value; }
        }

        public HSD HealthCareServicesDelivery
        {
            get { return _healthCareServicesDelivery; }
            set { _healthCareServicesDelivery = value; }
        }

        public List<CRC> ConditionsIndicator
        {
            get { return _conditionsIndicator; }
            set { _conditionsIndicator = value; }
        }

        public CL1 ClaimCodes
        {
            get { return _claimCodes; }
            set { _claimCodes = value; }
        }

        public CR1 AmbulanceCertification
        {
            get { return _ambulanceCertification; }
            set { _ambulanceCertification = value; }
        }

        public CR2 ChiropracticCertification
        {
            get { return _chiropracticCertification; }
            set { _chiropracticCertification = value; }
        }

        public CR4 EnteralOrParenteralTherapyCertification
        {
            get { return _enteralOrParenteralTherapyCertification; }
            set { _enteralOrParenteralTherapyCertification = value; }
        }

        public CR5 OxygenTherapyCertification
        {
            get { return _oxygenTherapyCertification; }
            set { _oxygenTherapyCertification = value; }
        }

        public CR6 HomeHealthCareCertification
        {
            get { return _homeHealthCareCertification; }
            set { _homeHealthCareCertification = value; }
        }

        public CR7 HomeHealthTreatmentPlanCertification
        {
            get { return _homeHealthTreatmentPlanCertification; }
            set { _homeHealthTreatmentPlanCertification = value; }
        }

        public CR8 PacemakerCertification
        {
            get { return _pacemakerCertification; }
            set { _pacemakerCertification = value; }
        }

        public List<PWK> Paperwork
        {
            get { return _paperwork; }
            set { _paperwork = value; }
        }

        public MSG MessageText
        {
            get { return _messageText; }
            set { _messageText = value; }
        }

        #endregion

        #region Methods

        public void CheckSegment(string segmentId, ref string strReturn)
        {
            switch (segmentId)
            {
                case "HL":
                    if (_hierarchicalLevel != null) strReturn += _hierarchicalLevel.ToString();
                    break;
                case "TRN":
                    if (_trace != null)
                    {
                        foreach (TRN t in _trace)
                        {
                            strReturn += t.ToString();
                        }
                    }
                    break;
                case "AAA":
                    if (_requestValidation != null)
                    {
                        foreach (AAA a in _requestValidation)
                        {
                            strReturn += a.ToString();
                        }
                    }
                    break;
                case "UM":
                    if (_healthCareServicesReviewInformation != null) strReturn += _healthCareServicesReviewInformation.ToString();
                    break;
                case "HCR":
                    if (_healthCareServicesReview != null) strReturn += _healthCareServicesReview.ToString();
                    break;
                case "REF":
                    if (_referenceInformation != null)
                    {
                        foreach (REF r in _referenceInformation)
                        {
                            strReturn += r.ToString();
                        }
                    }
                    break;
                case "DTP":
                    if (_dateOrTimeOrPeriod != null)
                    {
                        foreach (DTP d in _dateOrTimeOrPeriod)
                        {
                            strReturn += d.ToString();
                        }
                    }
                    break;
                case "HI":
                    if (_healthCareInformationCodes != null) strReturn += _healthCareInformationCodes.ToString();
                    break;
                case "SV1":
                    if (_professionalService != null) strReturn += _professionalService.ToString();
                    break;
                case "SV2":
                    if (_institutionalService != null) strReturn += _institutionalService.ToString();
                    break;
                case "SV3":
                    if (_dentalService != null) strReturn += _dentalService.ToString();
                    break;
                case "TOO":
                    if (_toothIdentification != null)
                    {
                        foreach (TOO t in _toothIdentification)
                        {
                            strReturn += t.ToString();
                        }
                    }
                    break;
                case "HSD":
                    if (_healthCareServicesDelivery != null) strReturn += _healthCareServicesDelivery.ToString();
                    break;
                case "CRC":
                    if (_conditionsIndicator != null)
                    {
                        foreach (CRC c in _conditionsIndicator)
                        {
                            strReturn += c.ToString();
                        }
                    }
                    break;
                case "CL1":
                    if (_claimCodes != null) strReturn += _claimCodes.ToString();
                    break;
                case "CR1":
                    if (_ambulanceCertification != null) strReturn += _ambulanceCertification.ToString();
                    break;
                case "CR2":
                    if (_chiropracticCertification != null) strReturn += _chiropracticCertification.ToString();
                    break;
                case "CR4":
                    if (_enteralOrParenteralTherapyCertification != null) strReturn += _enteralOrParenteralTherapyCertification.ToString();
                    break;
                case "CR5":
                    if (_oxygenTherapyCertification != null) strReturn += _oxygenTherapyCertification.ToString();
                    break;
                case "CR6":
                    if (_homeHealthCareCertification != null) strReturn += _homeHealthCareCertification.ToString();
                    break;
                case "CR7":
                    if (_homeHealthTreatmentPlanCertification != null) strReturn += _homeHealthTreatmentPlanCertification.ToString();
                    break;
                case "CR8":
                    if (_pacemakerCertification != null) strReturn += _pacemakerCertification.ToString();
                    break;
                case "PWK":
                    if (_paperwork != null)
                    {
                        foreach (PWK p in _paperwork)
                        {
                            strReturn += p.ToString();
                        }
                    }
                    break;
                case "MSG":
                    if (_messageText != null) strReturn += _messageText.ToString();
                    break;
                default:
                    break;
            }
        }

        public override string ToString()
        {
            string strReturn = string.Empty;

            if (_hierarchicalLevel != null) strReturn += _hierarchicalLevel.ToString();

            if (_trace != null)
            {
                foreach (TRN t in _trace)
                {
                    strReturn += t.ToString();
                }
            }

            if (_requestValidation != null)
            {
                foreach (AAA a in _requestValidation)
                {
                    strReturn += a.ToString();
                }
            }

            if (_healthCareServicesReviewInformation != null) strReturn += _healthCareServicesReviewInformation.ToString();

            if (_healthCareServicesReview != null) strReturn += _healthCareServicesReview.ToString();

            if (_referenceInformation != null)
            {
                foreach (REF r in _referenceInformation)
                {
                    strReturn += r.ToString();
                }
            }

            if (_dateOrTimeOrPeriod != null)
            {
                foreach (DTP d in _dateOrTimeOrPeriod)
                {
                    strReturn += d.ToString();
                }
            }

            if (_healthCareInformationCodes != null) strReturn += _healthCareInformationCodes.ToString();

            if (_professionalService != null) strReturn += _professionalService.ToString();

            if (_institutionalService != null) strReturn += _institutionalService.ToString();

            if (_dentalService != null) strReturn += _dentalService.ToString();

            if (_toothIdentification != null)
            {
                foreach (TOO t in _toothIdentification)
                {
                    strReturn += t.ToString();
                }
            }

            if (_healthCareServicesDelivery != null) strReturn += _healthCareServicesDelivery.ToString();

            if (_conditionsIndicator != null)
            {
                foreach (CRC c in _conditionsIndicator)
                {
                    strReturn += c.ToString();
                }
            }

            if (_claimCodes != null) strReturn += _claimCodes.ToString();

            if (_ambulanceCertification != null) strReturn += _ambulanceCertification.ToString();

            if (_chiropracticCertification != null) strReturn += _chiropracticCertification.ToString();

            if (_enteralOrParenteralTherapyCertification != null) strReturn += _enteralOrParenteralTherapyCertification.ToString();

            if (_oxygenTherapyCertification != null) strReturn += _oxygenTherapyCertification.ToString();

            if (_homeHealthCareCertification != null) strReturn += _homeHealthCareCertification.ToString();

            if (_homeHealthTreatmentPlanCertification != null) strReturn += _homeHealthTreatmentPlanCertification.ToString();

            if (_pacemakerCertification != null) strReturn += _pacemakerCertification.ToString();

            if (_paperwork != null)
            {
                foreach (PWK p in _paperwork)
                {
                    strReturn += p.ToString();
                }
            }

            if (_messageText != null) strReturn += _messageText.ToString();

            return strReturn;
        }

        #endregion
    }
}
