﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace X12Engine.BO.X12._278
{
   [Serializable()]
   public class EntityBase
   {

      private NM1 _individualOrOrganizationName;
      private List<REF> _referenceInformation;
      private N2 _additionalNameInformation;
      private N3 _partyLocation;
      private N4 _geographicLocation;
      private List<PER> _administrativeCommunicationsContact;
      private List<AAA> _requestValidation;
      private PRV _providerInformation;
      private DMG _demographicInformation;
      private INS _insuredBenefit;
      private List<DTP> _dateOrTimeOrPeriod;

      public EntityBase() { }

      public NM1 IndividualOrOrganizationName
      {
         get { return _individualOrOrganizationName; }
         set { _individualOrOrganizationName = value; }
      }

      public List<REF> ReferenceInformation
      {
         get { return _referenceInformation; }
         set { _referenceInformation = value; }
      }

      public N2 AdditionalNameInformation
      {
         get { return _additionalNameInformation; }
         set { _additionalNameInformation = value; }
      }

      public N3 PartyLocation
      {
         get { return _partyLocation; }
         set { _partyLocation = value; }
      }

      public N4 GeographicLocation
      {
         get { return _geographicLocation; }
         set { _geographicLocation = value; }
      }

      public List<PER> AdministrativeCommunicationsContact
      {
         get { return _administrativeCommunicationsContact; }
         set { _administrativeCommunicationsContact = value; }
      }

      public List<AAA> RequestValidation
      {
         get { return _requestValidation; }
         set { _requestValidation = value; }
      }

      public PRV ProviderInformation
      {
         get { return _providerInformation; }
         set { _providerInformation = value; }
      }

      public DMG DemographicInformation
      {
         get { return _demographicInformation; }
         set { _demographicInformation = value; }
      }

      public INS InsuredBenefit
      {
         get { return _insuredBenefit; }
         set { _insuredBenefit = value; }
      }

      public List<DTP> DateOrTimeOrPeriod
      {
         get { return _dateOrTimeOrPeriod; }
         set { _dateOrTimeOrPeriod = value; }
      }


      public void GetPatientInfo(ref string lastName, ref string firstName, ref string ssn, ref DateTime? dateOfBirth)
      {
         if (_individualOrOrganizationName != null)
         {
            lastName = _individualOrOrganizationName.NameLastOrOrganizationName;
            firstName = _individualOrOrganizationName.NameFirst;
            if (_individualOrOrganizationName.IdentificationCodeQualifier.Code == "MI")
            {
               // account for dashes
               ssn = _individualOrOrganizationName.IdentificationCode;
            }
         }

         if (_demographicInformation != null)
         {
            if (_demographicInformation.DateTimePeriodFormatQualifier != null && _demographicInformation.DateTimePeriodFormatQualifier.Code == "D8") //D8 = CCYYMMDD or yyyyMMdd
            {
               dateOfBirth = X12Engine.Utils.ParseDate(_demographicInformation.DateTimePeriod, "yyyyMMdd");
            }
         }

      }

      public override string ToString()
      {
         string strReturn = string.Empty;

         if (_individualOrOrganizationName != null) strReturn += _individualOrOrganizationName.ToString();

         if (_referenceInformation != null)
         {
               foreach (REF r in _referenceInformation)
               {
                  strReturn += r.ToString();
               }
         }

         if (_additionalNameInformation != null) strReturn += _additionalNameInformation.ToString();

         if (_partyLocation != null) strReturn += _partyLocation.ToString();

         if (_geographicLocation != null) strReturn += _geographicLocation.ToString();

         if (_administrativeCommunicationsContact != null)
         {
               foreach (PER p in _administrativeCommunicationsContact)
               {
                  strReturn += p.ToString();
               }
         }

         if (_requestValidation != null)
         {
               foreach (AAA a in _requestValidation)
               {
                  strReturn += a.ToString();
               }
         }

         if (_providerInformation != null) strReturn += _providerInformation.ToString();

         if (_demographicInformation != null) strReturn += _demographicInformation.ToString();

         if (_insuredBenefit != null) strReturn += _insuredBenefit.ToString();

         if (_dateOrTimeOrPeriod != null)
         {
               foreach (DTP d in _dateOrTimeOrPeriod)
               {
                  strReturn += d.ToString();
               }
         }

         return strReturn;
      }

   }
}
