﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;

namespace X12Engine
{
    public static class Utils
    {
       public static void LogEvent(string LogContent)
       {
          //System.Diagnostics.EventLog.WriteEntry("AATService", LogContent, System.Diagnostics.EventLogEntryType.Information);

          using (SqlConnection conn = new SqlConnection(Properties.Settings.Default.AATConnection))
          {
             conn.Open();

             using (SqlCommand cmd = new SqlCommand("INSERT INTO EventLog(LogContent) VALUES(@LogContent)", conn))
             {
                cmd.Parameters.AddWithValue("@LogContent", LogContent);
                cmd.ExecuteNonQuery();
             }
          }
       }


        public static DateTime? ParseDate(string element, string dateFormat)
        {
            if (string.IsNullOrEmpty(element)) return null;

            DateTime dateValue = DateTime.MinValue;

            if (string.IsNullOrEmpty(dateFormat))
            {
                dateFormat = "yyyyMMdd";
            }

            int dateFormatLength = (element.Length < dateFormat.Length) ? element.Length : dateFormat.Length;

            if (DateTime.TryParseExact(element.Substring(0, dateFormatLength), dateFormat, new CultureInfo("en-US"), DateTimeStyles.None, out dateValue))
                return dateValue;
            else
                return null;
        }

        public static DateTime? ParseTime(string element, string dateFormat, DateTime? date)
        {
            if (string.IsNullOrEmpty(element)) return null;

            DateTime dateValue = DateTime.MinValue;

            if (string.IsNullOrEmpty(dateFormat))
            {
                if (element.Length == 4) { dateFormat = "HHmm"; }
                if (element.Length == 5) { dateFormat = "HHmms"; }
                if (element.Length == 6) { dateFormat = "HHmmss"; }
                if (element.Length == 7) { dateFormat = "HHmmssf"; }
                if (element.Length == 8) { dateFormat = "HHmmssff"; }
            }

            int dateFormatLength = (element.Length < dateFormat.Length) ? element.Length : dateFormat.Length;

            if (DateTime.TryParseExact(element.Substring(0, dateFormatLength), dateFormat, new CultureInfo("en-US"), DateTimeStyles.NoCurrentDateDefault, out dateValue))
            {
                //if the date parameter is included, add the date portion to the time value
                if (date != null)
                {
                    DateTime dateTemp = new DateTime(date.Value.Year, date.Value.Month, date.Value.Day, dateValue.Hour, dateValue.Minute, dateValue.Second, dateValue.Millisecond);
                    dateValue = dateTemp;
                }

                return dateValue;
            }
            else
                return null;
        }

        public static decimal? ParseDecimal(string element)
        {
            decimal d = 0;

            if (decimal.TryParse(element, out d))
                return d;
            else
                return null;
        }

        public static int? ParseInt(string element)
        {
            int i = 0;

            if (int.TryParse(element, out i))
                return i;
            else
                return null;
        }

        public static string TerminateSegment(string segment)
        {
            while (segment.Substring(segment.Length - 1, 1) == Settings.Data_Element_Separator.ToString())
                segment = segment.Substring(0, segment.Length - 1);

            return segment + Settings.Segment_Terminator;
        }

        public static string TerminateComponent(string component)
        {
            while (component.Substring(component.Length - 1, 1) == Settings.Component_Element_Separator.ToString())
                component = component.Substring(0, component.Length - 1);

            return component;
        }

        public static bool IsEntitySegment(string segmentId)
        {
            switch (segmentId)
            {
                case "NM1":
                case "REF":
                case "N2":
                case "N3":
                case "N4":
                case "PER":
                case "AAA":
                case "PRV":
                case "DMG":
                case "INS":
                case "DTP":
                    return true;
                default:
                    return false;
            }
        }

        public static bool IsContactEntity(string code)
        {
            switch (code)
            {
                case "L5":
                    return true;
                default:
                    return false;
            }
        }

        public static bool IsTransportEntity(string code)
        {
            switch (code)
            {
                case "45":
                case "FS":
                case "ND":
                case "PW":
                case "R3":
                    return true;
                default:
                    return false;
            }
        }

        public static string[] Split(string input, string key)
        {
            string[] output = null;

            int index = input.IndexOf(key);

            while (index > 0)
            {
                string temp = input.Substring(0, index);
                AddToArray(ref output, temp);
                input = input.Remove(0, index);
                index = input.IndexOf(key, key.Length);
            }

            AddToArray(ref output, input);

            return output;
        }

        public static void AddToArray<t>(ref t[] arrObject, t obj)
        {
            if (arrObject == null)
            {
                arrObject = (t[])Array.CreateInstance(typeof(t), 1);
            }
            else
            {
                Array.Resize(ref arrObject, arrObject.Length + 1);
            }

            arrObject.SetValue(obj, arrObject.GetUpperBound(0));
        }

        public static bool Contains(string[] arrObject, string obj)
        {
            if (arrObject != null)
            {
                foreach (string s in arrObject)
                {
                    if (s == obj) return true;
                }
            }

            return false;
        }

        public static bool Contains(string[] arrObject, string obj, bool ignoreCase)
        {
            if (arrObject != null)
            {
                foreach (string s in arrObject)
                {
                    if (ignoreCase == true && s.ToUpper() == obj.ToUpper()) return true;
                    else if (s == obj) return true;
                }
            }

            return false;
        }

        public static int CountSegments(string message)
        {
            int objReturn = 0;

            if (!string.IsNullOrEmpty(message))
            {
                char[] chars = message.ToCharArray();
                if (chars != null)
                {
                    foreach (char c in chars)
                    {
                        if (c == Settings.Segment_Terminator) objReturn++;
                    }
                }
            }

            return objReturn;
        }
    }
}
